#!/usr/bin/env python
"""
Statistics module
"""

import numpy as np
from qtpy.QtWidgets import QApplication

from ibeatles.tools.tof_bin import BinAlgorithm, BinAutoMode, BinMode, StatisticsName, StatisticsRegion
from ibeatles.tools.tof_bin.utilities.get import Get
from ibeatles.tools.tof_bin.utilities.string import format_str
from ibeatles.tools.utilities import TimeSpectraKeys
from ibeatles.utilities import TO_ANGSTROMS_UNITS, TO_MICROS_UNITS
from ibeatles.utilities.table_handler import TableHandler


class Statistics:
    def __init__(self, parent=None):
        self.parent = parent

    def update(self):
        o_get = Get(parent=self.parent)
        statistics_dict = {}
        _info_statistics = {"list_file": None, "list_tof": None, "list_lambda": None}

        # check if we are looking for the auto or the manual bins
        bin_mode = o_get.bin_mode()

        if bin_mode == BinMode.auto:
            bin_auto_mode = o_get.bin_auto_mode()
            if bin_auto_mode == BinAutoMode.linear:
                list_bins = self.parent.linear_bins
            else:
                list_bins = self.parent.log_bins

        else:
            list_bins = self.parent.manual_bins

        o_table = TableHandler(table_ui=self.parent.ui.statistics_tableWidget)
        o_table.remove_all_rows()

        # if no bins to display, stop here
        if list_bins[TimeSpectraKeys.file_index_array] is None:
            return

        file_index_bins = list_bins[TimeSpectraKeys.file_index_array]
        tof_bins = list_bins[TimeSpectraKeys.tof_array]
        lambda_bins = list_bins[TimeSpectraKeys.lambda_array]

        mean_array_full = []
        median_array_full = []
        std_array_full = []
        min_array_full = []
        max_array_full = []

        mean_array_roi = []
        median_array_roi = []
        std_array_roi = []
        min_array_roi = []
        max_array_roi = []

        _row = 0
        for _bin_index, _bin in enumerate(file_index_bins):
            o_table.insert_empty_row(row=_row)

            o_table.insert_item(row=_row, column=0, value=str(_row), editable=False)

            if not _bin:
                mean_array_full.append(np.nan)
                median_array_full.append(np.nan)
                std_array_full.append(np.nan)
                min_array_full.append(np.nan)
                max_array_full.append(np.nan)

                mean_array_roi.append(np.nan)
                median_array_roi.append(np.nan)
                std_array_roi.append(np.nan)
                min_array_roi.append(np.nan)
                max_array_roi.append(np.nan)

                list_runs_formatted = "N/A"
                list_tof_formatted = "N/A"
                list_lambda_formatted = "N/A"
                str_mean = "N/A"
                str_median = "N/A"
                str_std = "N/A"
                str_min = "N/A"
                str_max = "N/A"

                # populate master statistics dict
                statistics_dict[_bin_index] = _info_statistics

            else:
                list_runs = file_index_bins[_bin_index]

                list_runs_formatted = format_str(
                    list_runs,
                    format_str="{:d}",
                    factor=1,
                    data_type=TimeSpectraKeys.file_index_array,
                )

                list_tof = tof_bins[_bin_index]
                list_tof_formatted = format_str(
                    list_tof,
                    format_str="{:.2f}",
                    factor=TO_MICROS_UNITS,
                    data_type=TimeSpectraKeys.tof_array,
                )

                list_lambda = lambda_bins[_bin_index]
                list_lambda_formatted = format_str(
                    list_lambda,
                    format_str="{:.3f}",
                    factor=TO_ANGSTROMS_UNITS,
                    data_type=TimeSpectraKeys.lambda_array,
                )

                list_tof_formatted_for_master_dict = [float(_tof * TO_MICROS_UNITS) for _tof in list_tof]
                list_lambda_formatted_for_master_dict = [float(_lambda * TO_ANGSTROMS_UNITS) for _lambda in list_lambda]

                statistics_dict[_bin_index] = {
                    "list_file": list_runs,
                    "list_tof": list_tof_formatted_for_master_dict,
                    "list_lambda": list_lambda_formatted_for_master_dict,
                }

                # calculate statistics
                _data_dict = self.extract_data_for_this_bin(list_runs=list_runs)

                full_image = _data_dict["full_image"]
                roi_of_image = _data_dict["roi_of_image"]

                # mean
                full_mean = np.mean(full_image)
                roi_mean = np.mean(roi_of_image)
                mean_array_full.append(full_mean)
                mean_array_roi.append(roi_mean)
                str_mean = f"{full_mean:.3f} ({roi_mean:.3f})"

                # median
                full_median = np.median(full_image)
                roi_median = np.median(roi_of_image)
                median_array_full.append(full_median)
                median_array_roi.append(roi_median)
                str_median = f"{full_median:.3f} ({roi_median:.3f})"

                # std
                full_std = np.std(full_image)
                roi_std = np.std(roi_of_image)
                std_array_full.append(full_std)
                std_array_roi.append(roi_std)
                str_std = f"{full_std:.3f} ({roi_std:.3f})"

                # min
                full_min = np.min(full_image)
                roi_min = np.min(roi_of_image)
                min_array_full.append(full_min)
                min_array_roi.append(roi_min)
                str_min = f"{full_min:.3f} ({roi_min:.3f})"

                # max
                full_max = np.max(full_image)
                roi_max = np.max(roi_of_image)
                max_array_full.append(full_max)
                max_array_roi.append(roi_max)
                str_max = f"{full_max:.3f} ({roi_max:.3f})"

            o_table.insert_item(row=_row, column=1, value=list_runs_formatted, editable=False)

            o_table.insert_item(row=_row, column=2, value=list_tof_formatted, editable=False)

            o_table.insert_item(row=_row, column=3, value=list_lambda_formatted, editable=False)

            o_table.insert_item(row=_row, column=4, value=str_mean, editable=False)

            o_table.insert_item(row=_row, column=5, value=str_median, editable=False)

            o_table.insert_item(row=_row, column=6, value=str_std, editable=False)

            o_table.insert_item(row=_row, column=7, value=str_min, editable=False)

            o_table.insert_item(row=_row, column=8, value=str_max, editable=False)

            _row += 1

        self.parent.current_stats[bin_mode] = {
            StatisticsName.mean: {
                StatisticsRegion.full: mean_array_full,
                StatisticsRegion.roi: mean_array_roi,
            },
            StatisticsName.median: {
                StatisticsRegion.full: median_array_full,
                StatisticsRegion.roi: median_array_roi,
            },
            StatisticsName.std: {
                StatisticsRegion.full: std_array_full,
                StatisticsRegion.roi: std_array_roi,
            },
            StatisticsName.min: {
                StatisticsRegion.full: min_array_full,
                StatisticsRegion.roi: min_array_roi,
            },
            StatisticsName.max: {
                StatisticsRegion.full: max_array_full,
                StatisticsRegion.roi: max_array_roi,
            },
        }

        self.parent.statistics_dict = statistics_dict

    def extract_data_for_this_bin(self, list_runs=None):
        """
        this method calculate the mean or median of full image and ROI selected of all the runs that
        belongs to that bin (list_runs are the run of that particular bin)

        :param list_runs:
        :return:
        """
        # retrieve statistics
        bin_roi = self.parent.bin_roi
        x0 = bin_roi["x0"]
        y0 = bin_roi["y0"]
        width = bin_roi["width"]
        height = bin_roi["height"]

        data_to_work_with = []
        for _run_index in list_runs:
            data_to_work_with.append(self.parent.images_array[_run_index])

        region_to_work_with = [_data[y0 : y0 + height, x0 : x0 + width] for _data in data_to_work_with]

        # how to add images
        o_get = Get(parent=self.parent)
        bin_method = o_get.bin_add_method()
        if bin_method == BinAlgorithm.mean:
            full_image_to_work_with = np.mean(data_to_work_with, axis=0)
            roi_image_to_work_with = np.mean(region_to_work_with, axis=0)
        elif bin_method == BinAlgorithm.median:
            full_image_to_work_with = np.median(data_to_work_with, axis=0)
            roi_image_to_work_with = np.median(region_to_work_with, axis=0)
        else:
            raise NotImplementedError("this method of adding the binned images is not supported!")

        return {
            "full_image": full_image_to_work_with,
            "roi_of_image": roi_image_to_work_with,
        }

    def plot_statistics(self):
        o_get = Get(parent=self.parent)
        bin_mode = o_get.bin_mode()

        if bin_mode == BinMode.auto:
            bin_auto_mode = o_get.bin_auto_mode()
            if bin_auto_mode == BinAutoMode.linear:
                list_bins = self.parent.linear_bins
            else:
                list_bins = self.parent.log_bins

        else:
            list_bins = self.parent.manual_bins

        # if no bins to display, stop here
        if list_bins[TimeSpectraKeys.file_index_array] is None:
            self.parent.statistics_plot.ax1.clear()
            QApplication.processEvents()
            return

        stats_requested = o_get.bin_statistics_plot_requested()

        stat_data_dict = self.parent.current_stats[bin_mode][stats_requested]

        full_array = stat_data_dict[StatisticsRegion.full]
        roi_array = stat_data_dict[StatisticsRegion.roi]

        self.parent.statistics_plot.ax1.clear()
        self.parent.statistics_plot.ax1.plot(full_array, ".", label="full image")
        self.parent.statistics_plot.ax1.plot(roi_array, "+", label="roi only")

        self.parent.statistics_plot.ax1.set_xlabel("Bin #")
        self.parent.statistics_plot.ax1.set_ylabel(stats_requested)
        self.parent.statistics_plot.ax1.legend(loc="upper right")

        self.parent.statistics_plot.draw()



#!/usr/bin/env python
"""
LoadPreviousSessionLauncher class
"""

import os

from qtpy.QtWidgets import QDialog

from ibeatles import load_ui
from ibeatles.session.load_previous_session_launcher_multiple_choice import (
    LoadPreviousSessionLauncherMultipleChoice,
)
from ibeatles.session.session_handler import SessionHandler
from ibeatles.utilities.get import Get


class LoadPreviousSessionLauncher(QDialog):
    def __init__(self, parent=None, config=None):
        self.parent = parent
        QDialog.__init__(self, parent=parent)
        ui_full_path = os.path.join(os.path.dirname(__file__), os.path.join("ui", "ui_load_previous_session.ui"))
        self.ui = load_ui(ui_full_path, baseinstance=self)
        self.setWindowTitle("Load previous session?")
        self.ui.pushButton.setFocus(True)

    def yes_clicked(self):
        self.close()
        o_session = SessionHandler(parent=self.parent)
        o_get = Get(parent=self.parent)
        full_config_file_name = o_get.get_automatic_config_file_name()
        o_session.load_from_file(config_file_name=full_config_file_name)

        if not o_session.load_successful:
            self.no_clicked()
            return

        list_tabs_to_load = o_session.get_tabs_to_load()
        if len(list_tabs_to_load) < 2:
            o_session.load_to_ui(tabs_to_load=list_tabs_to_load)
            self.parent.loading_from_config = False
        else:
            load_session_ui = LoadPreviousSessionLauncherMultipleChoice(
                parent=self.parent, list_tabs_to_load=list_tabs_to_load
            )
            load_session_ui.show()
        self.parent.check_log_file_size()

    def no_clicked(self):
        self.parent.session_dict = SessionHandler.session_dict
        self.close()
        self.parent.check_log_file_size()

    def reject(self):
        self.no_clicked()

    def closeEvent(self, event):
        self.close()



#!/usr/bin/env python
"""
SaveNormalizationTab class
"""

from loguru import logger

from ibeatles import DataType
from ibeatles.session import SessionSubKeys
from ibeatles.session.save_tab import SaveTab
from ibeatles.utilities.pyqrgraph import Pyqtgrah as PyqtgraphUtilities


class SaveNormalizationTab(SaveTab):
    def normalization(self):
        """record the ROI selected"""

        data_type = DataType.normalization
        list_roi = self.parent.list_roi[data_type]

        o_pyqt = PyqtgraphUtilities(
            parent=self.parent,
            image_view=self.parent.step2_ui["image_view"],
            data_type=data_type,
        )
        state = o_pyqt.get_state()
        o_pyqt.save_histogram_level()
        histogram = self.parent.image_view_settings[data_type]["histogram"]

        logger.info("Recording normalization information")
        logger.info(f" roi: {list_roi}")
        logger.info(f" state: {state}")
        logger.info(f" histogram: {histogram}")

        self.session_dict[data_type][SessionSubKeys.roi] = list_roi
        self.session_dict[data_type][SessionSubKeys.image_view_state] = state
        self.session_dict[data_type][SessionSubKeys.image_view_histogram] = histogram



#!/usr/bin/env python
"""
Rotate images
"""

import logging
import os
import shutil

import scipy
from qtpy.QtWidgets import QApplication, QMainWindow

from ibeatles import DataType, load_ui
from ibeatles.tools.rotate.event_handler import EventHandler as RotateEventHandler
from ibeatles.tools.rotate.initialization import Initialization
from ibeatles.tools.rotate.rotate_export_launcher import RotateExportLauncher


class RotateImages:
    def __init__(self, parent=None):
        self.parent = parent

        if self.parent.rotate_ui is None:
            rotate_ui = RotateImagesWindow(parent=parent)
            rotate_ui.show()
            self.parent.rotate_ui = rotate_ui
        else:
            self.parent.rotate_ui.setFocus()
            self.parent.rotate_ui.activateWindow()


class RotateImagesWindow(QMainWindow):
    grid_size = 100  # pixels

    integrated_image = None
    images_array = None

    histogram_level = None
    first_update = False

    list_tif_files = None
    image_size = {"width": None, "height": None}

    def __init__(self, parent=None):
        self.parent = parent
        QMainWindow.__init__(self, parent=parent)
        self.ui = load_ui("ui_rotateImages.ui", baseinstance=self)

        o_init = Initialization(parent=self)
        o_init.all()

    def select_folder_clicked(self):
        o_event = RotateEventHandler(parent=self, top_parent=self.parent)
        o_event.select_input_folder()
        o_event.load_data()
        o_event.display_rotated_images()
        o_event.check_widgets()

    def rotation_value_changed(self, rotation_value):
        o_event = RotateEventHandler(parent=self, top_parent=self.parent)
        o_event.display_rotated_images()
        o_event.check_widgets()

    def export_button_clicked(self):
        rotate_export_ui = RotateExportLauncher(parent=self, top_parent=self.parent)
        rotate_export_ui.show()

    # def save_and_use_clicked(self):
    #     logging.info("Rotating normalized images")
    #
    #     # select folder
    #     folder = os.path.dirname(self.parent.data_metadata[DataType.normalized]['folder'])
    #     output_folder = str(QFileDialog.getExistingDirectory(caption='Select Folder for Rotated Images ...',
    #                                                          directory=folder))
    #
    #     if not output_folder:
    #         logging.info(" User cancel rotating the images")
    #         return
    #
    #     QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
    #
    #     # create folder inside this selected folder
    #     rotation_value = self.ui.angle_horizontalSlider.value()
    #     if rotation_value < 0:
    #         rotation_value = f"minus{np.abs(rotation_value)}"
    #
    #     output_folder = os.path.join(output_folder, f"rotated_by_{rotation_value}degrees")
    #     FileHandler.make_or_reset_folder(output_folder)
    #
    #     self.rotate_and_save_all_images(target_folder=output_folder)
    #     self.reload_rotated_images()
    #     self.copy_time_spectra(target_folder=output_folder)
    #     QApplication.restoreOverrideCursor()
    #
    #     self.close()

    def copy_time_spectra(self, target_folder=None):
        time_spectra = self.parent.data_metadata[DataType.normalized]["time_spectra"]["filename"]
        target_filename = os.path.join(target_folder, os.path.basename(time_spectra))
        if os.path.exists(target_filename):
            os.remove(target_filename)
        self.parent.data_metadata[DataType.normalized]["time_spectra"]["filename"] = target_filename
        self.parent.data_metadata[DataType.normalized]["time_spectra"]["folder"] = os.path.dirname(target_filename)
        shutil.copyfile(time_spectra, target_filename)
        _folder_time_spectra = os.path.basename(os.path.abspath(target_folder))
        self.parent.ui.time_spectra_folder_2.setText(_folder_time_spectra)

    def reload_rotated_images(self):
        # update plot
        pass

    # def _save_image(self, filename='', data=[]):
    #     if os.path.exists(filename):
    #         os.remove(filename)
    #
    #     file_name, file_extension = os.path.splitext(filename)
    #     if file_extension.lower() in ['.tif', '.tiff']:
    #         FileHandler.make_tiff(data=data, filename=filename)
    #     elif file_extension.lower() == '.fits':
    #         FileHandler.make_fits(data=data, filename=filename)
    #     else:
    #         logging.info(f"file format {file_extension} not supported!")
    #         raise NotImplemented(f"file format {file_extension} not supported!")

    def rotate_and_save_all_images(self, target_folder=""):
        rotation_value = self.ui.angle_horizontalSlider.value()
        logging.info(f" rotation value: {rotation_value}")

        normalized_array = self.parent.data_metadata["normalized"]["data"]
        self.eventProgress.setValue(0)
        self.eventProgress.setMaximum(len(normalized_array))
        self.eventProgress.setVisible(True)

        rotated_normalized_array = []
        normalized_filename = self.parent.list_files[DataType.normalized]

        basefolder = os.path.basename(os.path.abspath(target_folder))
        self.parent.ui.normalized_folder.setText(basefolder)

        logging.info(f"-> base folder: {basefolder}")
        logging.info(f"-> target folder: {target_folder}")

        list_new_filename = []
        for _index, _data in enumerate(normalized_array):
            # rotate image
            rotated_data = scipy.ndimage.interpolation.rotate(_data, rotation_value)
            rotated_normalized_array.append(rotated_data)

            # save image
            new_filename = os.path.join(target_folder, normalized_filename[_index])
            list_new_filename.append(new_filename)

            self._save_image(filename=new_filename, data=rotated_data)

            self.eventProgress.setValue(_index + 1)
            QApplication.processEvents()

        self.parent.data_metadata["normalized"]["data"] = rotated_normalized_array
        self.parent.list_files[DataType.normalized] = list_new_filename
        self.eventProgress.setVisible(False)

        self.parent.normalized_list_selection_changed()

    def cancel_clicked(self):
        self.closeEvent(self)

    def closeEvent(self, event=None):
        self.parent.rotate_ui.close()
        self.parent.rotate_ui = None



#!/usr/bin/env python
"""
Export images module
"""

import logging
import os
import shutil

from NeuNorm.normalization import Normalization
from qtpy.QtWidgets import QFileDialog

from ibeatles.tools.tof_combine.utilities.get import Get
from ibeatles.tools.utilities import TimeSpectraKeys
from ibeatles.utilities.file_handler import FileHandler


class ExportImages:
    output_folder = None

    def __init__(self, parent=None, top_parent=None):
        self.parent = parent
        self.top_parent = top_parent

    def run(self):
        working_dir = self.parent.top_folder

        _folder = str(
            QFileDialog.getExistingDirectory(
                caption="Select Folder to ExportImages the Images",
                directory=working_dir,
                options=QFileDialog.ShowDirsOnly,
            )
        )

        if _folder == "":
            logging.info("User cancel export images!")
            return

        o_get = Get(parent=self.parent)
        nbr_folder = o_get.number_of_folders_we_want_to_combine()
        time_stamp = FileHandler.get_current_timestamp()
        output_folder = os.path.join(_folder, f"combined_{time_stamp}")
        self.output_folder = output_folder
        FileHandler.make_or_reset_folder(output_folder)
        logging.info(f"Combined images will be exported to {output_folder}!")

        # initialize progress bar
        self.parent.eventProgress.setMinimum(0)
        self.parent.eventProgress.setMaximum(nbr_folder - 1)
        self.parent.eventProgress.setValue(0)
        self.parent.eventProgress.setVisible(True)

        # export the arrays
        combine_arrays = self.parent.combine_data
        for _index, _array in enumerate(combine_arrays):
            short_file_name = f"image_{_index:04d}"
            o_norm = Normalization()
            o_norm.load(data=_array)
            o_norm.data["sample"]["file_name"][0] = short_file_name
            o_norm.export(folder=output_folder, data_type="sample", file_type="tiff")
            self.parent.eventProgress.setValue(_index + 1)

        # export time spectra file
        self.export_time_spectra_file()

        self.parent.eventProgress.setVisible(False)

    def export_time_spectra_file(self):
        """use the time spectra file from the first folder selected and export it to the output folder"""
        logging.info("Export time spectra file:")
        output_folder = self.output_folder

        # retrieve full path of the time spectra file from first folder selected
        full_path_time_spectra_file = self.parent.time_spectra[TimeSpectraKeys.file_name]

        logging.info(f" - time spectra file: {full_path_time_spectra_file}")
        logging.info(f" - to output folder: {output_folder}")

        # copy that spectra file to final location
        shutil.copy(full_path_time_spectra_file, output_folder)



class SessionKeys:
    material = "material"
    instrument = "instrument"
    bin = "bin"
    fitting = "fitting"
    settings = "settings"
    reduction = "reduction"


class MaterialMode:
    pre_defined = "pre_defined"
    custom_method1 = "custom method 1"
    custom_method2 = "custom method 2"


class ReductionDimension:
    twod = "2D"
    threed = "3D"


class ReductionType:
    box = "Box"
    gaussian = "Gaussian"


class SessionSubKeys:
    # instrument
    distance_source_detector = "distance source detector"
    detector_value = "detector value"
    beam_index = "beam index"

    # general
    list_files = "list files"
    current_folder = "current folder"
    list_files_selected = "list files selected"
    list_rois = "list rois"
    extension = "extension"

    # sample
    time_spectra_filename = "time spectra filename"

    # ob
    image_view_state = "image view state"
    image_view_histogram = "image view histogram"

    # material
    pre_defined_selected_element = "selected element in the pre-defined mode"
    pre_defined_selected_element_index = "index of pre-defined element"
    material_mode = "material mode"
    pre_defined = "pre_defined"
    custom_method1 = "custom method 1"
    custom_method2 = "custom method 2"
    custom_material_name = "name of the custom material"

    lattice = "lattice"
    crystal_structure = "crystal structure"
    crystal_structure_index = "index of crystal structure (0 for BCC, 1 for FCC)"
    index = "index"
    name = "name"
    user_defined = "user_defined"
    user_defined_fill_fields_with_element_index = "index of element used to fill the fields of method1"
    material_hkl_table = "hkl, d0 or lambda0 table"
    column_names = "names of the columns"

    # bin
    state = "state"
    roi = "roi"
    binning_line_view = "binning line view"
    nbr_row = "nbr row"
    nbr_column = "nbr column"
    bin_size = "bin size"

    # fitting
    ui_accessed = "ui_accessed"
    x_axis = "x_axis"
    xaxis = "xaxis"
    transparency = "transparency"
    ui = "ui"
    lambda_range_index = "lambda range index"
    bin_coordinates = "bin_coordinates"
    bragg_peak_threshold = "bragg peak threshold"

    # settings
    log_buffer_size = "log buffer size"
    config_version = "config version"

    # reduction
    activate = "activate"
    dimension = "dimension"
    size = "size"
    type = "type"
    process_order = "process order"

    # bin in TOF
    top_folder = "top_folder"
    list_working_folders = "list_working_folders"
    list_working_folders_status = "list_working_folders_status"
    version = "version"
    detector_offset = "detector_offset"
    combine_algorithm = "combine_algorithm"
    combine_roi = "combine_roi"
    sample_position = "sample_position"
    bin_mode = "bin_mode"
    bin_algorithm = "bin_algorithm"



#!/usr/bin/env python
"""View for the normalization settings."""

from qtpy.QtCore import Signal
from qtpy.QtWidgets import QDialog

from ibeatles.app.utils.ui_loader import load_ui
from ibeatles.core.config import (
    KernelSize,
    KernelType,
    MovingAverage,
    NormalizationConfig,
    ProcessOrder,
)


class NormalizationSettingsView(QDialog):
    """
    View class for the normalization settings dialog.

    This class represents the GUI for normalization settings, allowing users to
    configure moving average parameters, processing order, and sample backgrounds.
    """

    settings_changed = Signal()

    default_kernel_size = {"x": 3, "y": 3, "l": 3}
    default_kernel_size_label = {
        "3d": "y:{}  x:{}  λ:{}".format(default_kernel_size["y"], default_kernel_size["x"], default_kernel_size["l"]),
        "2d": f"y:{default_kernel_size['y']}  x:{default_kernel_size['x']}",
    }

    def __init__(self, parent=None):
        """
        Initialize the NormalizationSettingsView.

        Parameters
        ----------
        parent : QWidget, optional
            The parent widget.
        """
        super().__init__(parent)
        self.ui = load_ui("normalization_settings_view.ui", baseinstance=self)
        self.setWindowTitle("Normalization Settings")

    def ok_clicked(self):
        """Handle the OK button click."""
        self.settings_changed.emit()
        self.accept()

    def activate_moving_average_clicked(self):
        """Handle activation/deactivation of moving average."""
        state = self.ui.activate_moving_average_checkBox.isChecked()
        self.ui.dimension_groupBox.setEnabled(state)
        self.ui.size_groupBox.setEnabled(state)
        self.ui.type_groupBox.setEnabled(state)
        self.ui.processing_order_groupBox.setEnabled(state)
        self.settings_changed.emit()

    def dimension_radio_button_clicked(self):
        """Handle changes in kernel dimension selection."""
        is_3d_clicked = self.ui.kernel_dimension_3d_radioButton.isChecked()
        kernel_size = "3d" if is_3d_clicked else "2d"
        self.ui.kernel_size_default_label.setText(self.default_kernel_size_label[kernel_size])
        self.ui.kernel_size_custom_lambda_label.setVisible(is_3d_clicked)
        self.ui.kernel_size_custom_lambda_spinBox.setVisible(is_3d_clicked)
        self.settings_changed.emit()

    def size_radio_button_clicked(self):
        """Handle changes in kernel size selection."""
        is_default_clicked = self.ui.kernel_size_default_radioButton.isChecked()
        self.ui.kernel_size_default_label.setEnabled(is_default_clicked)
        self.ui.kernel_size_custom_y_label.setEnabled(not is_default_clicked)
        self.ui.kernel_size_custom_y_spinBox.setEnabled(not is_default_clicked)
        self.ui.kernel_size_custom_x_label.setEnabled(not is_default_clicked)
        self.ui.kernel_size_custom_x_spinBox.setEnabled(not is_default_clicked)
        self.ui.kernel_size_custom_lambda_label.setEnabled(not is_default_clicked)
        self.ui.kernel_size_custom_lambda_spinBox.setEnabled(not is_default_clicked)
        self.settings_changed.emit()

    def get_settings(self) -> NormalizationConfig:
        """
        Get the current settings from the UI.

        Returns
        -------
        NormalizationConfig
            A NormalizationConfig object containing the current settings.
        """
        is_3d = self.ui.kernel_dimension_3d_radioButton.isChecked()
        size = {
            "y": self.ui.kernel_size_custom_y_spinBox.value(),
            "x": self.ui.kernel_size_custom_x_spinBox.value(),
        }
        if is_3d:
            size["lambda"] = self.ui.kernel_size_custom_lambda_spinBox.value()

        return NormalizationConfig(
            moving_average=MovingAverage(
                active=self.ui.activate_moving_average_checkBox.isChecked(),
                dimension="3D" if is_3d else "2D",
                size=size,
                type=KernelType.gaussian if self.ui.kernel_type_gaussian_radioButton.isChecked() else KernelType.box,
            ),
            processing_order=ProcessOrder.moving_average_normalization
            if self.ui.processes_order_option1_radio_button.isChecked()
            else ProcessOrder.normalization_moving_average,
        )

    def set_settings(self, config: NormalizationConfig):
        """
        Set the UI elements based on the provided settings.

        Parameters
        ----------
        config : NormalizationConfig
            A NormalizationConfig object containing the settings to apply.
        """
        # Moving Average activation
        self.ui.activate_moving_average_checkBox.setChecked(config.moving_average.active)

        # Dimension
        is_3d = config.moving_average.dimension == "3D"
        self.ui.kernel_dimension_3d_radioButton.setChecked(is_3d)
        self.ui.kernel_dimension_2d_radioButton.setChecked(not is_3d)

        # Kernel Size
        default_size = KernelSize()
        is_default_size = (
            config.moving_average.size.y == default_size.y
            and config.moving_average.size.x == default_size.x
            and (config.moving_average.dimension == "2D" or config.moving_average.size.lambda_ == default_size.lambda_)
        )

        self.ui.kernel_size_default_radioButton.setChecked(is_default_size)
        self.ui.kernel_size_custom_radioButton.setChecked(not is_default_size)

        self.ui.kernel_size_custom_y_spinBox.setValue(config.moving_average.size.y)
        self.ui.kernel_size_custom_x_spinBox.setValue(config.moving_average.size.x)
        if is_3d:
            self.ui.kernel_size_custom_lambda_spinBox.setValue(config.moving_average.size.lambda_)

        # Kernel Type
        is_gaussian = config.moving_average.type == KernelType.gaussian
        self.ui.kernel_type_gaussian_radioButton.setChecked(is_gaussian)
        self.ui.kernel_type_box_radioButton.setChecked(not is_gaussian)

        # Processing Order
        is_ma_first = config.processing_order == ProcessOrder.moving_average_normalization
        self.ui.processes_order_option1_radio_button.setChecked(is_ma_first)
        self.ui.processes_order_option2_radio_button.setChecked(not is_ma_first)

        # Update UI state
        self.activate_moving_average_clicked()
        self.dimension_radio_button_clicked()
        self.size_radio_button_clicked()



#!/usr/bin/env python
"""
Session Handler
"""

import copy
import json

from loguru import logger
from qtpy.QtWidgets import QApplication, QFileDialog

from ibeatles import DataType
from ibeatles.fitting import FittingTabSelected
from ibeatles.fitting.kropff import BraggPeakInitParameters, KropffThresholdFinder
from ibeatles.fitting.kropff import SessionSubKeys as KropffSessionSubKeys
from ibeatles.fitting.march_dollase import SessionSubKeys as MarchSessionSubKeys
from ibeatles.session import MaterialMode, ReductionDimension, ReductionType, SessionKeys, SessionSubKeys
from ibeatles.session.general import General
from ibeatles.session.load_bin_tab import LoadBin
from ibeatles.session.load_fitting_tab import LoadFitting
from ibeatles.session.load_load_data_tab import LoadLoadDataTab
from ibeatles.session.load_normalization_tab import LoadNormalization
from ibeatles.session.load_normalized_tab import LoadNormalized
from ibeatles.session.save_bin_tab import SaveBinTab
from ibeatles.session.save_fitting_tab import SaveFittingTab
from ibeatles.session.save_load_data_tab import SaveLoadDataTab
from ibeatles.session.save_normalization_tab import SaveNormalizationTab
from ibeatles.session.save_normalized_tab import SaveNormalizedTab
from ibeatles.session.session_utilities import SessionUtilities
from ibeatles.utilities.get import Get
from ibeatles.utilities.status_message_config import (
    StatusMessageStatus,
    show_status_message,
)


class SessionHandler:
    config_file_name = ""
    load_successful = True

    session_dict = {
        SessionSubKeys.config_version: None,
        SessionSubKeys.log_buffer_size: 500,
        DataType.sample: {
            SessionSubKeys.list_files: None,
            SessionSubKeys.current_folder: None,
            SessionSubKeys.time_spectra_filename: None,
            SessionSubKeys.list_files_selected: None,
            SessionSubKeys.list_rois: None,
            SessionSubKeys.image_view_state: None,
            SessionSubKeys.image_view_histogram: None,
        },
        DataType.ob: {
            SessionSubKeys.list_files: None,
            SessionSubKeys.current_folder: None,
            SessionSubKeys.list_files_selected: None,
        },
        DataType.normalization: {
            SessionSubKeys.roi: None,
            SessionSubKeys.image_view_state: None,
            SessionSubKeys.image_view_histogram: None,
        },
        DataType.normalized: {
            SessionSubKeys.list_files: None,
            SessionSubKeys.current_folder: None,
            SessionSubKeys.time_spectra_filename: None,
            SessionSubKeys.list_files_selected: None,
            SessionSubKeys.list_rois: None,
            SessionSubKeys.image_view_state: None,
            SessionSubKeys.image_view_histogram: None,
        },
        SessionKeys.instrument: {
            SessionSubKeys.distance_source_detector: None,
            SessionSubKeys.beam_index: 0,
            SessionSubKeys.detector_value: None,
        },
        SessionKeys.material: {
            SessionSubKeys.material_mode: MaterialMode.pre_defined,
            SessionSubKeys.pre_defined: {SessionSubKeys.pre_defined_selected_element_index: 0},
            SessionSubKeys.custom_material_name: None,
            SessionSubKeys.custom_method1: {
                SessionSubKeys.lattice: None,
                SessionSubKeys.crystal_structure_index: 0,
                SessionSubKeys.material_hkl_table: None,
            },
            SessionSubKeys.custom_method2: {
                SessionSubKeys.material_hkl_table: None,
            },
        },
        SessionKeys.reduction: {
            SessionSubKeys.activate: True,
            SessionSubKeys.dimension: ReductionDimension.twod,
            SessionSubKeys.size: {
                "flag": "default",
                "y": 20,
                "x": 20,
                "l": 3,
            },
            SessionSubKeys.type: ReductionType.box,
            SessionSubKeys.process_order: "option1",
        },
        SessionKeys.bin: {
            SessionSubKeys.roi: None,  # ['name', x0, y0, width, height, bin_size]
            SessionSubKeys.nbr_row: None,  # number of bins in the vertical
            SessionSubKeys.nbr_column: None,  # number of bins in the horizontal
            SessionSubKeys.binning_line_view: {
                "pos": None,
                "adj": None,
                "line color": None,
            },
            SessionSubKeys.image_view_state: None,
            SessionSubKeys.image_view_histogram: None,
            SessionSubKeys.ui_accessed: False,
            SessionSubKeys.bin_size: 5,
        },
        DataType.fitting: {
            SessionSubKeys.lambda_range_index: None,
            SessionSubKeys.x_axis: None,
            SessionSubKeys.transparency: 50,
            SessionSubKeys.image_view_state: None,
            SessionSubKeys.image_view_histogram: None,
            SessionSubKeys.ui_accessed: False,
            SessionSubKeys.ui: {
                "splitter_2": None,
                "splitter": None,
                "splitter_3": None,
            },
            FittingTabSelected.march_dollase: {
                MarchSessionSubKeys.table_dictionary: None,
                MarchSessionSubKeys.plot_active_row_flag: True,
            },
            FittingTabSelected.kropff: {
                KropffSessionSubKeys.table_dictionary: None,
                KropffSessionSubKeys.high_tof: {
                    KropffSessionSubKeys.a0: "1",
                    KropffSessionSubKeys.b0: "1",
                    KropffSessionSubKeys.graph: "a0",
                },
                KropffSessionSubKeys.low_tof: {
                    KropffSessionSubKeys.ahkl: "1",
                    KropffSessionSubKeys.bhkl: "1",
                    KropffSessionSubKeys.graph: "ahkl",
                },
                KropffSessionSubKeys.bragg_peak: {
                    KropffSessionSubKeys.lambda_hkl: {
                        BraggPeakInitParameters.fix_value: "5e-8",
                        BraggPeakInitParameters.fix_flag: False,
                        BraggPeakInitParameters.range_from: "1e-8",
                        BraggPeakInitParameters.range_to: "1e-7",
                        BraggPeakInitParameters.range_step: "1e-8",
                    },
                    KropffSessionSubKeys.tau: {
                        BraggPeakInitParameters.fix_value: "1",
                        BraggPeakInitParameters.fix_flag: False,
                        BraggPeakInitParameters.range_from: "0.1",
                        BraggPeakInitParameters.range_to: "2",
                        BraggPeakInitParameters.range_step: "0.1",
                    },
                    KropffSessionSubKeys.sigma: {
                        BraggPeakInitParameters.fix_value: "1e-7",
                        BraggPeakInitParameters.fix_flag: False,
                        BraggPeakInitParameters.range_from: "1e-7",
                        BraggPeakInitParameters.range_to: "1e-1",
                        BraggPeakInitParameters.range_step: "10",
                    },
                    KropffSessionSubKeys.table_selection: "single",
                    KropffSessionSubKeys.graph: "lambda_hkl",
                },
                KropffSessionSubKeys.automatic_bragg_peak_threshold_finder: True,
                KropffSessionSubKeys.automatic_fitting_threshold_width: 5,
                KropffSessionSubKeys.automatic_bragg_peak_threshold_algorithm: KropffThresholdFinder.sliding_average,
                KropffSessionSubKeys.kropff_bragg_peak_good_fit_conditions: {
                    KropffSessionSubKeys.l_hkl_error: {
                        KropffSessionSubKeys.state: True,
                        KropffSessionSubKeys.value: 0.01,
                    },
                    KropffSessionSubKeys.t_error: {
                        KropffSessionSubKeys.state: False,
                        KropffSessionSubKeys.value: 0.01,
                    },
                    KropffSessionSubKeys.sigma_error: {
                        KropffSessionSubKeys.state: False,
                        KropffSessionSubKeys.value: 0.01,
                    },
                },
                KropffSessionSubKeys.kropff_lambda_settings: {
                    KropffSessionSubKeys.state: "fix",
                    KropffSessionSubKeys.fix: 5e-8,
                    KropffSessionSubKeys.range: [1e-8, 1e-7, 1e-8],
                },
                KropffSessionSubKeys.bragg_peak_row_rejections_conditions: {
                    KropffSessionSubKeys.l_hkl: {
                        KropffSessionSubKeys.less_than: {
                            KropffSessionSubKeys.state: True,
                            KropffSessionSubKeys.value: 0,
                        },
                        KropffSessionSubKeys.more_than: {
                            KropffSessionSubKeys.state: True,
                            KropffSessionSubKeys.value: 10,
                        },
                    },
                },
            },
        },
    }

    default_session_dict = copy.deepcopy(session_dict)

    def __init__(self, parent=None):
        logger.info("-> Saving current session before leaving the application")
        self.parent = parent

    def save_from_ui(self):
        self.session_dict[DataType.fitting][SessionSubKeys.ui_accessed] = self.parent.session_dict[DataType.fitting][
            SessionSubKeys.ui_accessed
        ]
        self.session_dict[SessionSubKeys.config_version] = self.parent.config[SessionSubKeys.config_version]
        self.session_dict[SessionSubKeys.log_buffer_size] = self.parent.session_dict[SessionSubKeys.log_buffer_size]

        self.session_dict = self.parent.session_dict

        # Load data tab
        o_save_load_data_tab = SaveLoadDataTab(parent=self.parent, session_dict=self.session_dict)
        o_save_load_data_tab.sample()
        o_save_load_data_tab.ob()
        o_save_load_data_tab.instrument()
        o_save_load_data_tab.material()
        self.session_dict = o_save_load_data_tab.session_dict

        # save normalization
        o_save_normalization = SaveNormalizationTab(parent=self.parent, session_dict=self.session_dict)
        o_save_normalization.normalization()
        self.session_dict = o_save_normalization.session_dict

        # save normalized
        o_save_normalized = SaveNormalizedTab(parent=self.parent, session_dict=self.session_dict)
        o_save_normalized.normalized()
        self.session_dict = o_save_normalized.session_dict

        # save bin
        o_save_bin = SaveBinTab(parent=self.parent, session_dict=self.session_dict)
        o_save_bin.bin()
        self.session_dict = o_save_bin.session_dict

        # save fitting
        o_save_fitting = SaveFittingTab(parent=self.parent, session_dict=self.session_dict)
        o_save_fitting.fitting()
        self.session_dict = o_save_fitting.session_dict

        self.parent.session_dict = self.session_dict

    def load_to_ui(self, tabs_to_load=None):
        if not self.load_successful:
            return

        logger.info(f"Automatic session tabs to load: {tabs_to_load}")

        try:
            o_general = General(parent=self.parent)
            o_general.settings()

            if DataType.sample in tabs_to_load:
                # load data tab
                o_load = LoadLoadDataTab(parent=self.parent)
                o_load.sample()
                o_load.ob()
                self.parent.load_data_tab_changed(tab_index=0)
                self.parent.ui.sample_ob_splitter.setSizes([20, 450])

                # load normalization tab
                o_norm = LoadNormalization(parent=self.parent)
                o_norm.roi()
                o_norm.check_widgets()
                o_norm.image_settings()

            o_load = LoadLoadDataTab(parent=self.parent)
            o_load.instrument()
            o_load.material()

            self.parent.material_tab_changed()  # to make sure the hkl and lambda are correctly saved

            if DataType.normalized in tabs_to_load:
                # load normalized tab
                o_normalized = LoadNormalized(parent=self.parent)
                o_normalized.all()

            if DataType.bin in tabs_to_load:
                # load bin tab
                o_bin = LoadBin(parent=self.parent)
                o_bin.all()

            if DataType.fitting in tabs_to_load:
                # load fitting
                o_fit = LoadFitting(parent=self.parent)
                o_fit.table_dictionary()

            o_util = SessionUtilities(parent=self.parent)
            if DataType.normalized in tabs_to_load:
                o_util.jump_to_tab_of_data_type(DataType.normalized)

            if DataType.bin in tabs_to_load:
                o_util.jump_to_tab_of_data_type(DataType.bin)

            if DataType.fitting in tabs_to_load:
                o_util.jump_to_tab_of_data_type(DataType.fitting)

            show_status_message(
                parent=self.parent,
                message=f"Loaded {self.config_file_name}",
                status=StatusMessageStatus.ready,
                duration_s=10,
            )

        except FileNotFoundError:
            show_status_message(
                parent=self.parent,
                message="One of the data file could not be located. Aborted loading session!",
                status=StatusMessageStatus.error,
                duration_s=10,
            )
            logger.info("Loading session aborted! FileNotFoundError raised!")
            self.parent.session_dict = SessionHandler.session_dict

        except ValueError:
            show_status_message(
                parent=self.parent,
                message="One of the data file raised an error. Aborted loading session!",
                status=StatusMessageStatus.error,
                duration_s=10,
            )
            logger.info("Loading session aborted! ValueError raised!")
            self.parent.session_dict = SessionHandler.session_dict

    def automatic_save(self):
        o_get = Get(parent=self.parent)
        full_config_file_name = o_get.get_automatic_config_file_name()
        self.save_to_file(config_file_name=full_config_file_name)

    def save_to_file(self, config_file_name=None):
        if config_file_name is None:
            config_file_name = QFileDialog.getSaveFileName(
                self.parent,
                caption="Select session file name ...",
                directory=self.parent.default_path[DataType.sample],
                filter="session (*.json)",
                initialFilter="session",
            )

            QApplication.processEvents()
            config_file_name = config_file_name[0]

        if config_file_name:
            output_file_name = config_file_name
            session_dict = self.session_dict

            with open(output_file_name, "w") as json_file:
                json.dump(session_dict, json_file)

            show_status_message(
                parent=self.parent,
                message=f"Session saved in {config_file_name}",
                status=StatusMessageStatus.ready,
                duration_s=10,
            )
            logger.info(f"Saving configuration into {config_file_name}")

    def load_from_file(self, config_file_name=None):
        self.parent.loading_from_config = True

        if config_file_name is None:
            config_file_name = QFileDialog.getOpenFileName(
                self.parent,
                directory=self.parent.default_path[DataType.sample],
                caption="Select session file ...",
                filter="session (*.json)",
                initialFilter="session",
            )
            QApplication.processEvents()
            config_file_name = config_file_name[0]

        if config_file_name:
            config_file_name = config_file_name
            self.config_file_name = config_file_name
            show_status_message(
                parent=self.parent,
                message=f"Loading {config_file_name} ...",
                status=StatusMessageStatus.ready,
            )

            with open(config_file_name, "r") as read_file:
                session_to_save = json.load(read_file)
                if session_to_save.get(SessionSubKeys.config_version, None) is None:
                    logger.info("Session file is out of date!")
                    logger.info(f"-> expected version: {self.parent.config[SessionSubKeys.config_version]}")
                    logger.info("-> session version: Unknown!")
                    self.load_successful = False
                elif (
                    session_to_save[SessionSubKeys.config_version] == self.parent.config[SessionSubKeys.config_version]
                ):
                    self.parent.session_dict = session_to_save
                    logger.info(f"Loaded from {config_file_name}")
                else:
                    logger.info("Session file is out of date!")
                    logger.info(f"-> expected version: {self.parent.config[SessionSubKeys.config_version]}")
                    logger.info(f"-> session version: {session_to_save[SessionSubKeys.config_version]}")
                    self.load_successful = False

                if not self.load_successful:
                    show_status_message(
                        parent=self.parent,
                        message=f"{config_file_name} not loaded! (check log for more information)",
                        status=StatusMessageStatus.ready,
                        duration_s=10,
                    )

        else:
            self.load_successful = False
            show_status_message(
                parent=self.parent,
                message=f"{config_file_name} not loaded! (check log for more information)",
                status=StatusMessageStatus.ready,
                duration_s=10,
            )

    def get_tabs_to_load(self):
        session_dict = self.parent.session_dict
        list_tabs_to_load = []
        if session_dict[DataType.sample][SessionSubKeys.list_files]:
            list_tabs_to_load.append(DataType.sample)
        if session_dict[DataType.normalized][SessionSubKeys.list_files]:
            list_tabs_to_load.append(DataType.normalized)
        if session_dict[DataType.bin][SessionSubKeys.ui_accessed]:
            list_tabs_to_load.append(DataType.bin)
        if session_dict[DataType.fitting][SessionSubKeys.ui_accessed]:
            list_tabs_to_load.append(DataType.fitting)

        return list_tabs_to_load



#!/usr/bin/env python
"""
FittingParametersViewerEditorLauncher class for handling the fitting parameters viewer editor launcher.
"""

import numpy as np
from loguru import logger
from qtpy import QtCore, QtGui
from qtpy.QtWidgets import QApplication, QMainWindow, QMenu

from ibeatles import load_ui
from ibeatles.fitting.filling_table_handler import FillingTableHandler
from ibeatles.fitting.kropff import SessionSubKeys
from ibeatles.fitting.kropff.fitting_parameters_viewer_editor_handler import (
    FittingParametersViewerEditorHandler,
)
from ibeatles.fitting.kropff.get import Get
from ibeatles.utilities.array_utilities import calculate_median
from ibeatles.utilities.bins import (
    convert_bins_to_keys,
    create_list_of_bins_from_selection,
    create_list_of_surrounding_bins,
)
from ibeatles.utilities.table_handler import TableHandler


class FittingParametersViewerEditorLauncher:
    def __init__(self, grand_parent=None, parent=None):
        self.grand_parent = grand_parent

        if self.grand_parent.kropff_fitting_parameters_viewer_editor_ui is None:
            set_variables_window = FittingParametersViewerEditor(grand_parent=grand_parent, parent=parent)
            self.grand_parent.kropff_fitting_parameters_viewer_editor_ui = set_variables_window
            set_variables_window.show()
        else:
            self.grand_parent.kropff_fitting_parameters_viewer_editor_ui.setFocus()
            self.grand_parent.kropff_fitting_parameters_viewer_editor_ui.activateWindow()


class FittingParametersViewerEditor(QMainWindow):
    advanced_mode = True
    nbr_column = -1
    nbr_row = -1

    def __init__(self, grand_parent=None, parent=None):
        self.grand_parent = grand_parent
        self.parent = parent
        QMainWindow.__init__(self, parent=grand_parent)
        self.ui = load_ui("ui_fittingVariablesKropff.ui", baseinstance=self)
        self.setWindowTitle("Check/Set Variables")

        self.kropff_table_dictionary = self.grand_parent.kropff_table_dictionary

        self.init_widgets()
        self.init_table()
        self.fill_table()

    def eventFilter(self, object, event):
        if event.type() == QtCore.QEvent.WindowActivate:
            self.update_table()
        return False

    def init_table(self):
        fitting_selection = self.grand_parent.fitting_selection

        # print(fitting_selection)
        nbr_row = fitting_selection["nbr_row"]
        nbr_column = fitting_selection["nbr_column"]

        self.nbr_column = nbr_column
        self.nbr_row = nbr_row

        # selection table
        self.ui.variable_table.setColumnCount(nbr_column)
        self.ui.variable_table.setRowCount(nbr_row)

        # set size of cells
        value = int(self.ui.advanced_selection_cell_size_slider.value())
        self.selection_cell_size_changed(value)

    def init_widgets(self):
        self.ui.lambda_hkl_button.setText("\u03bb\u2095\u2096\u2097")
        self.ui.tau_button.setText("\u03c4")
        self.ui.sigma_button.setText("\u03c3")

    def fill_table(self):
        QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
        o_get = Get(parent=self)
        variable_selected = o_get.variable_selected()
        o_handler = FittingParametersViewerEditorHandler(grand_parent=self.grand_parent, parent=self)
        o_handler.populate_table_with_variable(variable=variable_selected)
        QApplication.restoreOverrideCursor()

    def selection_cell_size_changed(self, value):
        nbr_row = self.ui.variable_table.rowCount()
        nbr_column = self.ui.variable_table.columnCount()

        for _row in np.arange(nbr_row):
            self.ui.variable_table.setRowHeight(_row, value)
            # self.ui.colorscale_table.setRowHeight(_row, value)

        for _col in np.arange(nbr_column):
            self.ui.variable_table.setColumnWidth(_col, value)
            # self.ui.colorscale_table.setColumnWidth(_col, value)

    def update_table(self):
        QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
        o_get = Get(parent=self)
        variable_selected = o_get.variable_selected()
        o_handler = FittingParametersViewerEditorHandler(grand_parent=self.grand_parent, parent=self)
        o_handler.populate_table_with_variable(variable=variable_selected)

        # o_filling_table = FillingTableHandler(grand_parent=self.grand_parent,
        #                                       parent=self.parent)
        # self.grand_parent.fitting_ui.ui.value_table.blockSignals(True)
        # o_filling_table.fill_table()
        # self.grand_parent.fitting_ui.ui.value_table.blockSignals(False)
        QApplication.restoreOverrideCursor()

    def apply_new_value_to_selection(self):
        o_get = Get(parent=self)
        variable_selected = o_get.variable_selected()
        selection = self.grand_parent.fitting_set_variables_ui.ui.variable_table.selectedRanges()
        o_handler = FittingParametersViewerEditorHandler(grand_parent=self.grand_parent)
        new_variable = float(str(self.grand_parent.fitting_set_variables_ui.ui.new_value_text_edit.text()))
        o_handler.set_new_value_to_selected_bins(
            selection=selection,
            variable_name=variable_selected,
            variable_value=new_variable,
            table_nbr_row=self.nbr_row,
        )
        self.grand_parent.fitting_set_variables_ui.ui.new_value_text_edit.setText("")
        o_filling_table = FillingTableHandler(grand_parent=self.grand_parent, parent=self.parent)
        self.grand_parent.fitting_ui.ui.value_table.blockSignals(True)
        o_filling_table.fill_table()
        self.grand_parent.fitting_ui.ui.value_table.blockSignals(False)

    def variable_table_right_click(self, position):
        o_variable = VariableTableHandler(
            grand_grand_parent=self.grand_parent,
            grand_parent=self.parent,
            parent=self,
        )
        o_variable.right_click(position=position)

    def variable_table_cell_changed(self, row, column):
        o_handler = FittingParametersViewerEditorHandler(parent=self, grand_parent=self.grand_parent)
        o_handler.variable_cell_manual_changed(row=row, column=column)

    def save_and_quit_clicked(self):
        logger.info("Saving fitting parameters back into fitting tab!")
        self.grand_parent.kropff_table_dictionary = self.kropff_table_dictionary
        o_fill = FillingTableHandler(parent=self.parent, grand_parent=self.grand_parent)
        o_fill.fill_kropff_bragg_peak_table()
        self.close()

    def closeEvent(self, event=None):
        self.grand_parent.kropff_fitting_parameters_viewer_editor_ui = None


class VariableTableHandler:
    nbr_row = None
    nbr_column = None

    def __init__(self, grand_grand_parent=None, grand_parent=None, parent=None):
        self.grand_grand_parent = grand_grand_parent
        self.grand_parent = grand_parent
        self.parent = parent

        self.nbr_column = self.parent.nbr_column
        self.nbr_row = self.parent.nbr_row

    def right_click(self, position=None):
        menu = QMenu(self.grand_parent)

        # checking selection and if any, enabled buttons
        o_table = TableHandler(table_ui=self.parent.ui.variable_table)
        current_selection = o_table.get_selection()

        state_button = True if len(current_selection) > 0 else False

        _lock = menu.addAction("Lock Selection")
        _lock.setEnabled(state_button)
        _unlock = menu.addAction("Unlock and replace by median of surrounding pixels")
        _unlock.setEnabled(state_button)

        action = menu.exec_(QtGui.QCursor.pos())

        if action == _lock:
            self.lock_selection()
        elif action == _unlock:
            self.unlock_selection()
            self.replace_by_median_of_surrounding_pixels()

    def replace_by_median_of_surrounding_pixels(self):
        """
        replace all the pixels selected by the median of the surrounding pixels and
        automatically lock that cell
        """
        o_table = TableHandler(table_ui=self.parent.ui.variable_table)
        all_selection = o_table.get_selection()

        table_dictionary = self.parent.kropff_table_dictionary

        logger.info("replace by median of surrounding pixels")

        for _selection in all_selection:
            top_row = _selection.topRow()
            bottom_row = _selection.bottomRow()
            left_column = _selection.leftColumn()
            right_column = _selection.rightColumn()

            # make individual list of bins to work on
            list_bins = create_list_of_bins_from_selection(
                top_row=top_row,
                bottom_row=bottom_row,
                left_column=left_column,
                right_column=right_column,
            )

            logger.info(f"-> list_bins: {list_bins}")
            for central_bin in list_bins:
                [central_key] = convert_bins_to_keys(list_of_bins=[central_bin], full_bin_height=self.nbr_row)

                if self.parent.kropff_table_dictionary[central_key][SessionSubKeys.lock]:
                    logger.info(f"-> bin #{central_key} is locked and won't be modified!")
                    # we don't do anything if the cell is locked !
                    continue

                surrounding_bins = create_list_of_surrounding_bins(
                    central_bin=central_bin,
                    full_bin_width=self.nbr_column,
                    full_bin_height=self.nbr_row,
                )

                surrounding_keys = convert_bins_to_keys(list_of_bins=surrounding_bins, full_bin_height=self.nbr_row)

                list_lambda_value = []
                list_tau_value = []
                list_sigma_value = []

                list_lambda_error = []
                list_tau_error = []
                list_sigma_error = []

                for _key in surrounding_keys:
                    list_lambda_value.append(table_dictionary[_key][SessionSubKeys.lambda_hkl]["val"])
                    list_tau_value.append(table_dictionary[_key][SessionSubKeys.tau]["val"])
                    list_sigma_value.append(table_dictionary[_key][SessionSubKeys.sigma]["val"])

                    list_lambda_error.append(table_dictionary[_key][SessionSubKeys.lambda_hkl]["err"])
                    list_tau_error.append(table_dictionary[_key][SessionSubKeys.tau]["err"])
                    list_sigma_error.append(table_dictionary[_key][SessionSubKeys.sigma]["err"])

                new_lambda_value = calculate_median(array_of_value=list_lambda_value)
                new_lambda_error = calculate_median(array_of_value=list_lambda_error)

                new_tau_value = calculate_median(array_of_value=list_tau_value)
                new_tau_error = calculate_median(array_of_value=list_tau_error)

                new_sigma_value = calculate_median(array_of_value=list_sigma_value)
                new_sigma_error = calculate_median(array_of_value=list_sigma_error)

                self.parent.kropff_table_dictionary[central_key][SessionSubKeys.lambda_hkl]["val"] = new_lambda_value
                self.parent.kropff_table_dictionary[central_key][SessionSubKeys.lambda_hkl]["err"] = new_lambda_error

                self.parent.kropff_table_dictionary[central_key][SessionSubKeys.tau]["val"] = new_tau_value
                self.parent.kropff_table_dictionary[central_key][SessionSubKeys.tau]["err"] = new_tau_error

                self.parent.kropff_table_dictionary[central_key][SessionSubKeys.sigma]["val"] = new_sigma_value
                self.parent.kropff_table_dictionary[central_key][SessionSubKeys.sigma]["err"] = new_sigma_error

                self.parent.kropff_table_dictionary[central_key][SessionSubKeys.lock] = True

        # refresh table
        self.parent.update_table()

        # clear selection
        o_table = TableHandler(table_ui=self.parent.ui.variable_table)
        o_table.select_everything(False)

    def set_fixed_status_of_selection(self, state=True):
        selection = self.grand_parent.fitting_set_variables_ui.ui.variable_table.selectedRanges()
        table_dictionary = self.grand_parent.march_table_dictionary
        nbr_row = self.grand_parent.fitting_set_variables_ui.nbr_row

        o_get = Get(parent=self.parent)
        variable_selected = o_get.variable_selected()

        for _select in selection:
            _left_column = _select.leftColumn()
            _right_column = _select.rightColumn()
            _top_row = _select.topRow()
            _bottom_row = _select.bottomRow()
            for _row in np.arange(_top_row, _bottom_row + 1):
                for _col in np.arange(_left_column, _right_column + 1):
                    _index = _row + _col * nbr_row
                    table_dictionary[str(_index)][variable_selected]["fixed"] = state

            # remove selection markers
            self.grand_parent.fitting_set_variables_ui.ui.variable_table.setRangeSelected(_select, False)

        self.grand_parent.march_table_dictionary = table_dictionary
        self.grand_parent.fitting_set_variables_ui.update_table()

    def fixed_selection(self):
        self.set_fixed_status_of_selection(state=True)

    def unfixed_selection(self):
        self.set_fixed_status_of_selection(state=False)

    def lock_selection(self):
        self.change_state_of_bins(name="lock", state=True)
        self.parent.update_table()
        # o_table = TableHandler(table_ui=self.parent.ui.variable_table)
        # o_table.select_everything(False)

    def unlock_selection(self):
        self.change_state_of_bins(name="lock", state=False)
        self.parent.update_table()
        # o_table = TableHandler(table_ui=self.parent.ui.variable_table)
        # o_table.select_everything(False)

    def change_state_of_bins(self, name="lock", state=True):
        o_table = TableHandler(table_ui=self.parent.ui.variable_table)
        all_selection = o_table.get_selection()
        # table_dictionary = self.parent.kropff_table_dictionary
        logger.info("Changing lock state of selection")

        for _selection in all_selection:
            top_row = _selection.topRow()
            bottom_row = _selection.bottomRow()
            left_column = _selection.leftColumn()
            right_column = _selection.rightColumn()

            # make individual list of bins to work on
            list_bins = create_list_of_bins_from_selection(
                top_row=top_row,
                bottom_row=bottom_row,
                left_column=left_column,
                right_column=right_column,
            )

            list_keys = convert_bins_to_keys(list_of_bins=list_bins, full_bin_height=self.nbr_row)

            for _key in list_keys:
                self.parent.kropff_table_dictionary[_key][SessionSubKeys.lock] = state



#!/usr/bin/env python
"""
Fitting Handler
"""

import copy

import numpy as np
import pyqtgraph as pg
from qtpy.QtWidgets import QGraphicsRectItem

from ibeatles import DataType
from ibeatles.fitting import FittingTabSelected, KropffTabSelected, lock_color, selected_color
from ibeatles.fitting.filling_table_handler import FillingTableHandler
from ibeatles.fitting.selected_bin_handler import SelectedBinsHandler
from ibeatles.session import SessionKeys, SessionSubKeys
from ibeatles.utilities import colors
from ibeatles.utilities.array_utilities import get_min_max_xy
from ibeatles.utilities.pyqrgraph import Pyqtgrah as PyqtgraphUtilities


class FittingHandler:
    kropff_table_dictionary_template = {
        "yaxis": None,
        "xaxis": None,
        "fitted": {
            KropffTabSelected.high_tof: {"xaxis": None, "yaxis": None},
            KropffTabSelected.low_tof: {"xaxis": None, "yaxis": None},
            KropffTabSelected.bragg_peak: {"xaxis": None, "yaxis": None},
        },
        "locked_item": None,
        "selected": False,
        "lock": False,
        "rejected": False,
        "active": False,
        "a0": {"val": np.nan, "err": np.nan},
        "b0": {"val": np.nan, "err": np.nan},
        "ahkl": {"val": np.nan, "err": np.nan},
        "bhkl": {"val": np.nan, "err": np.nan},
        "lambda_hkl": {"val": np.nan, "err": np.nan},
        "tau": {"val": np.nan, "err": np.nan},
        "sigma": {"val": np.nan, "err": np.nan},
        "bragg peak threshold": {"left": None, "right": None},
    }

    def __init__(self, grand_parent=None, parent=None):
        self.parent = parent
        self.grand_parent = grand_parent

    def display_image(self, data=[]):
        o_pyqt = PyqtgraphUtilities(
            parent=self.grand_parent,
            image_view=self.grand_parent.image_view,
            data_type=DataType.normalized,
        )
        _state = o_pyqt.get_state()
        o_pyqt.save_histogram_level()

        if len(data) > 0:
            self.parent.data = data
            self.parent.image_view.setImage(data)
        else:
            if len(self.grand_parent.data_metadata["normalized"]["data_live_selection"]) > 0:
                data = np.array(self.grand_parent.data_metadata["normalized"]["data_live_selection"])
                if len(data) == 0:
                    return
                else:
                    self.parent.image_view.setImage(data)
                    self.parent.data = data

        o_pyqt.set_state(_state)
        o_pyqt.reload_histogram_level()

    def display_roi(self):
        if len(np.array(self.grand_parent.data_metadata["normalized"]["data_live_selection"])) == 0:
            return

        pos = self.grand_parent.binning_line_view["pos"]
        adj = self.grand_parent.binning_line_view["adj"]
        lines = self.grand_parent.binning_line_view["pen"]

        if pos is None:
            return

        self.grand_parent.there_is_a_roi = True

        # define new transparency of roi
        transparency = self.parent.slider.value()
        self.grand_parent.fitting_transparency_slider_value = transparency
        lines = colors.set_alpha_value(lines=lines, transparency=transparency)

        if self.parent.line_view_fitting:
            self.parent.image_view.removeItem(self.parent.line_view_fitting)

        line_view_fitting = pg.GraphItem()
        self.parent.line_view_fitting = line_view_fitting
        self.parent.image_view.addItem(line_view_fitting)
        self.parent.line_view = line_view_fitting

        self.parent.line_view.setData(pos=pos, adj=adj, pen=lines, symbol=None, pxMode=False)

    def fill_table(self):
        if len(np.array(self.grand_parent.data_metadata["normalized"]["data_live_selection"])) == 0:
            return

        if not self.grand_parent.there_is_a_roi:
            return

        self.create_table_dictionary()

        if self.grand_parent.table_loaded_from_session:
            self.initialize_parameters_from_session()

        o_fill_table = FillingTableHandler(grand_parent=self.grand_parent, parent=self.parent)
        o_fill_table.fill_table()

        if self.grand_parent.table_loaded_from_session:
            self.display_locked_active_bins()

    def display_locked_active_bins(self):
        o_bin_handler = SelectedBinsHandler(parent=self.parent, grand_parent=self.grand_parent)
        o_bin_handler.update_bins_locked()
        o_bin_handler.update_bins_selected()
        o_bin_handler.update_bragg_edge_plot()
        self.parent.min_or_max_lambda_manually_changed()
        self.parent.check_status_widgets()

    def initialize_parameters_from_session(self):
        # self.initialize_marche_dollase_parameters_from_session()
        self.initialize_kropff_parameters_from_session()

    def initialize_kropff_parameters_from_session(self):
        session_table_dictionary = self.grand_parent.session_dict[DataType.fitting][FittingTabSelected.kropff][
            "table dictionary"
        ]
        table_dictionary = self.grand_parent.kropff_table_dictionary

        for _row in session_table_dictionary.keys():
            _entry = session_table_dictionary[_row]

            table_dictionary[_row]["bragg peak threshold"] = _entry["bragg_peak_threshold"]
            table_dictionary[_row]["a0"] = _entry["a0"]
            table_dictionary[_row]["b0"] = _entry["b0"]
            table_dictionary[_row]["ahkl"] = _entry["ahkl"]
            table_dictionary[_row]["bhkl"] = _entry["bhkl"]
            table_dictionary[_row]["lambda_hkl"] = _entry["lambda_hkl"]
            table_dictionary[_row]["tau"] = _entry["tau"]
            table_dictionary[_row]["sigma"] = _entry["sigma"]

        self.grand_parent.kropff_table_dictionary = table_dictionary

    def initialize_marche_dollase_parameters_from_session(self):
        session_table_dictionary = self.grand_parent.session_dict[SessionKeys.fitting][
            FittingTabSelected.march_dollase
        ]["table dictionary"]
        table_dictionary = self.grand_parent.march_table_dictionary

        for _row in session_table_dictionary.keys():
            _entry = session_table_dictionary[_row]
            table_dictionary[_row]["lock"] = _entry["lock"]
            table_dictionary[_row]["active"] = _entry["active"]
            table_dictionary[_row]["fitting_confidence"] = _entry["fitting_confidence"]
            table_dictionary[_row]["d_spacing"] = _entry["d_spacing"]
            table_dictionary[_row]["sigma"] = _entry["sigma"]
            table_dictionary[_row]["alpha"] = _entry["alpha"]
            table_dictionary[_row]["a1"] = _entry["a1"]
            table_dictionary[_row]["a2"] = _entry["a2"]
            table_dictionary[_row]["a5"] = _entry["a5"]
            table_dictionary[_row]["a6"] = _entry["a6"]

        lambda_range = self.grand_parent.session_dict["fitting"]["lambda range index"]
        if lambda_range:
            [lambda_min_index, lambda_max_index] = self.grand_parent.session_dict["fitting"]["lambda range index"]
            x_axis = self.grand_parent.session_dict["fitting"]["x_axis"]

            lambda_min = x_axis[lambda_min_index]
            lambda_max = x_axis[lambda_max_index]

            self.parent.ui.lambda_min_lineEdit.setText("{:4.2f}".format(lambda_min))
            self.parent.ui.lambda_max_lineEdit.setText("{:4.2f}".format(lambda_max))
            self.grand_parent.fitting_bragg_edge_linear_selection = [
                lambda_min_index,
                lambda_max_index,
            ]

        transparency = self.grand_parent.session_dict["fitting"]["transparency"]
        self.parent.ui.slider.setValue(transparency)

        self.grand_parent.display_active_row_flag = self.grand_parent.session_dict[SessionKeys.fitting][
            FittingTabSelected.march_dollase
        ]["plot active row flag"]
        self.parent.ui.active_bins_button.setChecked(self.grand_parent.display_active_row_flag)
        self.parent.ui.locked_bins_button.setChecked(not self.grand_parent.display_active_row_flag)

        self.grand_parent.march_table_dictionary = table_dictionary
        self.grand_parent.table_loaded_from_session = None

    def create_table_dictionary(self):
        """
        this will define the corner position and index of each cell
        """
        # if len(np.array(self.grand_parent.data_metadata['normalized']['data_live_selection'])) == 0:
        #     return
        #
        # if not self.grand_parent.march_table_dictionary == {}:
        #     return

        bin_size = self.grand_parent.session_dict[DataType.bin][SessionSubKeys.roi][5]
        pos = self.grand_parent.binning_line_view["pos"]

        # calculate outside real edges of bins
        min_max_xy = get_min_max_xy(pos)

        from_x = min_max_xy["x"]["min"]
        to_x = min_max_xy["x"]["max"]

        from_y = min_max_xy["y"]["min"]
        to_y = min_max_xy["y"]["max"]

        march_table_dictionary = {}
        kropff_table_dictionary = {}
        _index = 0
        _index_col = 0
        _index_row = 0

        for _x in np.arange(from_x, to_x, bin_size):
            _index_row = 0
            for _y in np.arange(from_y, to_y, bin_size):
                _str_index = str(_index)

                kropff_table_dictionary[_str_index] = copy.deepcopy(self.kropff_table_dictionary_template)
                kropff_table_dictionary[_str_index]["bin_coordinates"] = {
                    "x0": _x,
                    "x1": _x + bin_size,
                    "y0": _y,
                    "y1": _y + bin_size,
                }
                kropff_table_dictionary[_str_index]["row_index"] = _index_row
                kropff_table_dictionary[_str_index]["column_index"] = _index_col
                kropff_table_dictionary[_str_index]["selected_item"] = None
                kropff_table_dictionary[_str_index]["lock"] = False
                kropff_table_dictionary[_str_index]["rejected"] = False

                # create the box to show when bin is selected
                selection_box = QGraphicsRectItem(_x, _y, bin_size, bin_size)
                selection_box.setPen(pg.mkPen(selected_color["pen"]))
                selection_box.setBrush(pg.mkBrush(selected_color["brush"]))
                kropff_table_dictionary[_str_index]["selected_item"] = selection_box

                march_table_dictionary[_str_index] = {
                    "bin_coordinates": {
                        "x0": _x,
                        "x1": _x + bin_size,
                        "y0": _y,
                        "y1": _y + bin_size,
                    },
                    "selected_item": None,
                    "locked_item": None,
                    "row_index": _index_row,
                    "column_index": _index_col,
                    "selected": False,
                    "lock": False,
                    "active": False,
                    "fitting_confidence": np.nan,
                    "d_spacing": {"val": np.nan, "err": np.nan, "fixed": False},
                    "sigma": {"val": np.nan, "err": np.nan, "fixed": False},
                    "intensity": {"val": np.nan, "err": np.nan, "fixed": False},
                    "alpha": {"val": np.nan, "err": np.nan, "fixed": False},
                    "a1": {"val": np.nan, "err": np.nan, "fixed": False},
                    "a2": {"val": np.nan, "err": np.nan, "fixed": False},
                    "a5": {"val": np.nan, "err": np.nan, "fixed": False},
                    "a6": {"val": np.nan, "err": np.nan, "fixed": False},
                }

                # march_table_dictionary[_str_index]['bin_coordinates']['x0'] = _x
                # march_table_dictionary[_str_index]['bin_coordinates']['x1'] = _x + bin_size
                # march_table_dictionary[_str_index]['bin_coordinates']['y0'] = _y
                # march_table_dictionary[_str_index]['bin_coordinates']['y1'] = _y + bin_size

                # create the box to show when bin is selected
                selection_box = QGraphicsRectItem(_x, _y, bin_size, bin_size)
                selection_box.setPen(pg.mkPen(selected_color["pen"]))
                selection_box.setBrush(pg.mkBrush(selected_color["brush"]))
                march_table_dictionary[_str_index]["selected_item"] = selection_box

                # create the box to show when bin is locked
                lock_box = QGraphicsRectItem(_x, _y, bin_size, bin_size)
                lock_box.setPen(pg.mkPen(lock_color["pen"]))
                lock_box.setBrush(pg.mkBrush(lock_color["brush"]))
                march_table_dictionary[_str_index]["locked_item"] = lock_box

                _index += 1
                _index_row += 1

            _index_col += 1

        self.grand_parent.march_table_dictionary = march_table_dictionary
        self.grand_parent.kropff_table_dictionary = kropff_table_dictionary

        self.grand_parent.fitting_selection["nbr_row"] = _index_row
        self.grand_parent.fitting_selection["nbr_column"] = _index_col

        self.grand_parent.session_dict[SessionKeys.bin][SessionSubKeys.nbr_row] = _index_row
        self.grand_parent.session_dict[SessionKeys.bin][SessionSubKeys.nbr_column] = _index_col



#!/usr/bin/env python
"""
SaveLoadDataTab class
"""

import os

from loguru import logger

from ibeatles import DataType
from ibeatles.session import SessionKeys, SessionSubKeys
from ibeatles.session.save_tab import SaveTab
from ibeatles.utilities.gui_handler import GuiHandler
from ibeatles.utilities.pyqrgraph import Pyqtgrah as PyqtgraphUtilities


class SaveLoadDataTab(SaveTab):
    def sample(self):
        """record all the parameters of the Load Data tab / Sample accordion tab"""

        data_type = DataType.sample

        list_files = self.parent.list_files[data_type]
        if len(list_files) == 0:
            return

        current_folder = self.parent.data_metadata[data_type]["folder"]
        time_spectra_filename = self.parent.data_metadata[data_type]["time_spectra"]["filename"]
        list_files_selected = [int(index) for index in self.parent.list_file_selected[data_type]]
        list_roi = self.parent.list_roi[data_type]
        o_pyqt = PyqtgraphUtilities(
            parent=self.parent,
            image_view=self.parent.ui.image_view,
            data_type=data_type,
        )
        state = o_pyqt.get_state()
        o_pyqt.save_histogram_level()
        histogram = self.parent.image_view_settings[data_type]["histogram"]

        extension = os.path.splitext(list_files[0])

        logger.info("Recording parameters of Load Data / Sample")
        logger.info(f" len(list files) = {len(list_files)}")
        logger.info(f" current folder: {current_folder}")
        logger.info(f" time spectra filename: {time_spectra_filename}")
        logger.info(f" list files selected: {list_files_selected}")
        logger.info(f" len(list rois): {len(list_roi)}")
        logger.info(f" state: {state}")
        logger.info(f" histogram: {histogram}")
        logger.info(f" extension: {extension}")

        self.session_dict[data_type][SessionSubKeys.list_files] = list_files
        self.session_dict[data_type][SessionSubKeys.current_folder] = current_folder
        self.session_dict[data_type][SessionSubKeys.time_spectra_filename] = time_spectra_filename
        self.session_dict[data_type][SessionSubKeys.extension] = extension
        self.session_dict[data_type][SessionSubKeys.list_files_selected] = list_files_selected
        self.session_dict[data_type][SessionSubKeys.list_rois] = list_roi
        self.session_dict[data_type][SessionSubKeys.image_view_state] = state
        self.session_dict[data_type][SessionSubKeys.image_view_histogram] = histogram

    def ob(self):
        """record all the parameters of the Load Data tab / ob accordion tab"""

        data_type = DataType.ob

        list_files = self.parent.list_files[data_type]
        if len(list_files) == 0:
            return

        current_folder = self.parent.data_metadata[data_type]["folder"]
        list_files_selected = [int(index) for index in self.parent.list_file_selected[data_type]]
        list_roi = self.parent.list_roi[data_type]
        o_pyqt = PyqtgraphUtilities(
            parent=self.parent,
            image_view=self.parent.ui.ob_image_view,
            data_type=data_type,
        )
        state = o_pyqt.get_state()
        o_pyqt.save_histogram_level()
        histogram = self.parent.image_view_settings[data_type]["histogram"]

        logger.info("Recording parameters of Load Data / OB")
        logger.info(f" len(list files) = {len(list_files)}")
        logger.info(f" current folder: {current_folder}")
        logger.info(f" list files selected: {list_files_selected}")
        logger.info(f" len(list rois): {len(list_roi)}")
        logger.info(f" state: {state}")
        logger.info(f" histogram: {histogram}")

        self.session_dict[data_type][SessionSubKeys.list_files] = list_files
        self.session_dict[data_type][SessionSubKeys.current_folder] = current_folder
        self.session_dict[data_type][SessionSubKeys.list_files_selected] = list_files_selected
        self.session_dict[data_type][SessionSubKeys.list_rois] = list_roi
        self.session_dict[data_type][SessionSubKeys.image_view_state] = state
        self.session_dict[data_type][SessionSubKeys.image_view_histogram] = histogram

    def instrument(self):
        """record the settings of the instrument such as offset, distance source/detector ..."""

        list_ui = {
            "distance": self.parent.ui.distance_source_detector,
            "beam": self.parent.ui.beam_rate,
            "detector": self.parent.ui.detector_offset,
        }

        o_gui = GuiHandler(parent=self.parent)
        distance_value = o_gui.get_text(ui=list_ui["distance"])
        detector_value = o_gui.get_text(ui=list_ui["detector"])
        beam_index = o_gui.get_index_selected(ui=list_ui["beam"])

        logger.info("Recording instrument")
        logger.info(f" distance source detector: {distance_value}")
        logger.info(f" detector value: {detector_value}")
        logger.info(f" beam index: {beam_index}")

        self.session_dict[SessionKeys.instrument][SessionSubKeys.distance_source_detector] = distance_value
        self.session_dict[SessionKeys.instrument][SessionSubKeys.detector_value] = detector_value
        self.session_dict[SessionKeys.instrument][SessionSubKeys.beam_index] = beam_index

    def material(self):
        """record the material settings (element selected, full list, crystal structure, lattice"""

        # pre-defined
        pre_defined_material_index = self.parent.ui.pre_defined_list_of_elements.currentIndex()
        o_gui = GuiHandler(parent=self.parent)
        material_mode = o_gui.get_material_active_tab()

        # custom
        custom_material_name = self.parent.ui.user_defined_element_name.text()

        ## method1
        index_element = self.parent.ui.user_defined_list_of_elements.currentIndex()
        lattice_value = self.parent.ui.method1_lattice_value_2.text()
        crystal_structure_index = self.parent.ui.method1_crystal_value_2.currentIndex()
        method1_table, column1_names = GuiHandler.collect_table_data(table_ui=self.parent.ui.method1_tableWidget)

        ## method 2
        method2_table, column2_names = GuiHandler.collect_table_data(table_ui=self.parent.method2_tableWidget)

        self.session_dict[SessionKeys.material] = {
            SessionSubKeys.material_mode: material_mode,
            SessionSubKeys.pre_defined: {SessionSubKeys.pre_defined_selected_element_index: pre_defined_material_index},
            SessionSubKeys.custom_material_name: custom_material_name,
            SessionSubKeys.custom_method1: {
                SessionSubKeys.user_defined_fill_fields_with_element_index: index_element,
                SessionSubKeys.lattice: lattice_value,
                SessionSubKeys.crystal_structure_index: crystal_structure_index,
                SessionSubKeys.material_hkl_table: method1_table,
                SessionSubKeys.column_names: column1_names,
            },
            SessionSubKeys.custom_method2: {
                SessionSubKeys.material_hkl_table: method2_table,
                SessionSubKeys.column_names: column2_names,
            },
        }

        logger.info("Recording Material")
        logger.info(f" pre-defined material index: {pre_defined_material_index}")
        logger.info(f" custom material name: {custom_material_name}")
        logger.info(f" index of element used to fill fields: {index_element}")
        logger.info(f" custom method1 lattice: {lattice_value}")
        logger.info(f" custom method1 crystal structure index: {crystal_structure_index}")
        logger.info(f" custom method1 table: {method1_table}")
        logger.info(f" custom method2 table: {method2_table}")



#!/usr/bin/env python
"""
Event handler for the step6
"""

import numpy as np

from ibeatles.fitting.kropff.get import Get as GetKropff
from ibeatles.step6.display import Display
from ibeatles.step6.get import Get


class EventHandler:
    def __init__(self, parent=None, grand_parent=None):
        self.parent = parent
        self.grand_parent = grand_parent

        self.width = self.parent.image_size["width"]
        self.height = self.parent.image_size["height"]

        o_get = GetKropff(grand_parent=self.grand_parent)
        self.nbr_row = o_get.nbr_row()
        self.nbr_column = o_get.nbr_column()

    def process_data(self):
        self.calculate_d_array()
        self.calculate_strain_mapping_array()

    def calculate_d_array(self):
        d_array = np.empty((self.height, self.width))
        d_array[:] = np.nan

        compact_d_array = np.empty((self.nbr_row, self.nbr_column))

        d_dict = {}

        top_left_corner_of_roi = [self.height, self.width]

        kropff_table_dictionary = self.grand_parent.kropff_table_dictionary
        for _row_index in kropff_table_dictionary.keys():
            _row_entry = kropff_table_dictionary[_row_index]

            bin_coordinates = _row_entry["bin_coordinates"]
            x0 = bin_coordinates["x0"]
            x1 = bin_coordinates["x1"]
            y0 = bin_coordinates["y0"]
            y1 = bin_coordinates["y1"]

            row_index = _row_entry["row_index"]
            column_index = _row_entry["column_index"]

            if x0 < top_left_corner_of_roi[1]:
                top_left_corner_of_roi[1] = x0

            if y0 < top_left_corner_of_roi[0]:
                top_left_corner_of_roi[0] = y0

            lambda_hkl = _row_entry["lambda_hkl"]["val"]
            lambda_hkl_err = _row_entry["lambda_hkl"]["err"]
            if lambda_hkl_err is None:
                lambda_hkl_err = np.sqrt(lambda_hkl)

            d_array[y0:y1, x0:x1] = float(lambda_hkl) / 2.0
            compact_d_array[row_index, column_index] = float(lambda_hkl) / 2.0

            d_dict[_row_index] = {
                "val": float(lambda_hkl) / 2.0,
                "err": float(lambda_hkl_err) / 2.0,
            }

        self.parent.d_array = d_array
        self.parent.compact_d_array = compact_d_array
        self.parent.d_dict = d_dict
        self.parent.top_left_corner_of_roi = top_left_corner_of_roi

    def calculate_strain_mapping_array(self):
        d_array = self.parent.d_array
        compact_d_array = self.parent.compact_d_array

        o_get = Get(parent=self.parent)
        d0 = o_get.active_d0()

        self.parent.strain_mapping_array = (d_array - d0) / d0
        self.parent.compact_strain_mapping_array = (compact_d_array - d0) / d0

    def min_max_changed(self):
        o_display = Display(parent=self.parent, grand_parent=self.grand_parent)
        o_display.run()

    def interpolation_cmap_method_changed(self):
        o_display = Display(parent=self.parent, grand_parent=self.grand_parent)
        o_display.run()



#!/usr/bin/env python
"""
Add element editor
"""

from collections import OrderedDict

import numpy as np
from qtpy import QtCore
from qtpy.QtWidgets import QDialog

from ibeatles import Material, load_ui
from ibeatles.step1.plot import Step1Plot
from ibeatles.utilities.bragg_edge_element_handler import BraggEdgeElementCalculator
from ibeatles.utilities.check import is_float, is_int
from ibeatles.utilities.gui_handler import GuiHandler
from ibeatles.utilities.table_handler import TableHandler


class AddElement(object):
    def __init__(self, parent=None):
        self.parent = parent

    def run(self):
        _interface = AddElementInterface(parent=self.parent)
        _interface.show()
        self.parent.add_element_editor_ui = _interface


class AddElementInterface(QDialog):
    new_element = {}

    def __init__(self, parent=None):
        self.parent = parent

        QDialog.__init__(self, parent=parent)
        self.ui = load_ui("ui_addElement.ui", baseinstance=self)
        self.setWindowTitle("Add Element Editor")
        self.ui.element_name_error.setVisible(False)
        self.check_add_widget_state()

    def element_name_changed(self, current_value):
        self.check_add_widget_state()

    def lattice_changed(self, current_value):
        self.check_add_widget_state()

    def check_add_widget_state(self):
        self.ui.error_message.setText("")
        current_element_name = str(self.ui.element_name.text())
        if current_element_name.strip() == "":
            self.ui.add.setEnabled(False)
            self.ui.error_message.setText("Provide an element name!")
            return

        if self.ui.method1_radioButton.isChecked():  # method 1
            lattice_value = self.ui.lattice.text()
            if lattice_value.strip() == "":
                self.ui.add.setEnabled(False)
                self.ui.error_message.setText("Lattice value is missing!")
                return

            if not is_float(lattice_value):
                self.ui.add.setEnabled(False)
                self.ui.error_message.setText("Lattice must be a number")
                return

            list_element_root = self.parent.ui.list_of_elements.findText(
                current_element_name, QtCore.Qt.MatchCaseSensitive
            )
            if not (list_element_root == -1):  # element already there
                self.ui.element_name_error.setVisible(True)
                self.ui.add.setEnabled(False)
                self.ui.error_message.setText("Element name already used!")
                return

            else:
                self.ui.element_name_error.setVisible(False)
                self.ui.add.setEnabled(True)
                return

        else:  # method 2
            # at least one entry in the table
            o_table = TableHandler(table_ui=self.ui.tableWidget)
            nbr_row = o_table.row_count()
            at_least_one_row_valid = False
            for _row in np.arange(nbr_row):
                h = o_table.get_item_str_from_cell(row=_row, column=0)
                k = o_table.get_item_str_from_cell(row=_row, column=1)
                l = o_table.get_item_str_from_cell(row=_row, column=2)  # noqa E741
                d0 = o_table.get_item_str_from_cell(row=_row, column=3)

                if (
                    ((h is None) or h == "")
                    and ((k is None) or k == "")
                    and ((l is None) or l == "")
                    and ((d0 is None) or d0 == "")
                ):
                    continue

                elif h is None:
                    self.ui.add.setEnabled(False)
                    self.ui.error_message.setText("missing h value!")
                    return

                elif k is None:
                    self.ui.add.setEnabled(False)
                    self.ui.error_message.setText("missing k value!")
                    return

                elif l is None:
                    self.ui.add.setEnabled(False)
                    self.ui.error_message.setText("missing l value!")
                    return

                elif d0 is None:
                    self.ui.add.setEnabled(False)
                    self.ui.error_message.setText("missing d0 value!")
                    return

                if (h.strip() == "") and (k.strip() == "") and (l.strip() == "") and (d0.strip() == ""):
                    continue

                if not is_int(h):
                    self.ui.add.setEnabled(False)
                    self.ui.error_message.setText("h must be an integer!")
                    return

                if not is_int(k):
                    self.ui.add.setEnabled(False)
                    self.ui.error_message.setText("k must be an integer!")
                    return

                if not is_int(l):
                    self.ui.add.setEnabled(False)
                    self.ui.error_message.setText("l must be an integer!")
                    return

                if not is_float(d0):
                    self.ui.add.setEnabled(False)
                    self.ui.error_message.setText("d0 must be an integer!")
                    return

                at_least_one_row_valid = True

            self.ui.add.setEnabled(at_least_one_row_valid)

            if not at_least_one_row_valid:
                self.ui.error_message.setText("Define at least 1 row of entries!")

    def method2_table_changed(self):
        self.check_add_widget_state()

    def retrieve_metadata(self):
        o_gui = GuiHandler(parent=self)
        element_name = o_gui.get_text(ui=self.ui.element_name)

        if self.ui.method1_radioButton.isChecked():  # method 1
            user_defined = False
            method_used = Material.via_lattice_and_crystal_structure
            lattice = o_gui.get_text(ui=self.ui.lattice)
            crystal_structure = o_gui.get_text_selected(ui=self.ui.crystal_structure)

            # calculate the hkl and d0 here
            o_calculator = BraggEdgeElementCalculator(
                element_name=element_name,
                lattice_value=lattice,
                crystal_structure=crystal_structure,
            )
            o_calculator.run()
            selected_element_bragg_edges_array = o_calculator.lambda_array
            selected_element_hkl_array = o_calculator.hkl_array

            hkl_d0_dict = OrderedDict()
            for _row_index in np.arange(len(selected_element_hkl_array)):
                hkl_list = selected_element_hkl_array[_row_index]
                h = hkl_list[0]
                k = hkl_list[1]
                l = hkl_list[2]  # noqa E741

                _lambda_value = selected_element_bragg_edges_array[_row_index]
                d0 = _lambda_value / 2.0

                hkl_d0_dict[int(_row_index)] = {"h": h, "k": k, "l": l, "d0": d0}

        else:
            lattice = None
            method_used = Material.via_d0
            user_defined = True
            crystal_structure = None

            o_table = TableHandler(table_ui=self.ui.tableWidget)
            nbr_row = o_table.row_count()
            hkl_d0_dict = OrderedDict()
            _row_index = 0
            for _row in np.arange(nbr_row):
                h = o_table.get_item_str_from_cell(row=_row, column=0)
                if h == "":
                    continue

                k = o_table.get_item_str_from_cell(row=_row, column=1)
                l = o_table.get_item_str_from_cell(row=_row, column=2)  # noqa E741
                d0 = o_table.get_item_str_from_cell(row=_row, column=3)
                hkl_d0_dict[int(_row_index)] = {"h": h, "k": k, "l": l, "d0": d0}
                _row_index += 1

        self.new_element = {
            Material.element_name: element_name,
            Material.lattice: lattice,
            Material.crystal_structure: crystal_structure,
            Material.hkl_d0: hkl_d0_dict,
            Material.user_defined: user_defined,
            Material.method_used: method_used,
        }

    def add_element_to_list_of_elements_widgets(self):
        _element = self.new_element
        self.parent.ui.list_of_elements.blockSignals(True)
        self.parent.ui.list_of_elements.addItem(_element[Material.element_name])
        nbr_element = self.parent.ui.list_of_elements.count()
        self.parent.ui.list_of_elements.setCurrentIndex(nbr_element - 1)
        self.parent.ui.lattice_parameter.setText(_element[Material.lattice])
        self.parent.ui.list_of_elements.blockSignals(False)

    def save_new_element_to_local_list(self):
        _new_element = self.new_element

        _new_entry = {
            Material.lattice: _new_element[Material.lattice],
            Material.crystal_structure: _new_element[Material.crystal_structure],
            Material.hkl_d0: _new_element[Material.hkl_d0],
            Material.method_used: _new_element[Material.method_used],
        }

        self.parent.user_defined_bragg_edge_list[_new_element[Material.element_name]] = _new_entry
        self.parent.local_bragg_edge_list[_new_element[Material.element_name]] = _new_entry

    def method_changed(self):
        is_method1_activated = self.ui.method1_radioButton.isChecked()
        self.ui.method1_groupBox.setEnabled(is_method1_activated)
        self.ui.method2_groupBox.setEnabled(not is_method1_activated)
        self.check_add_widget_state()

    def add_clicked(self):
        self.retrieve_metadata()
        self.save_new_element_to_local_list()
        self.add_element_to_list_of_elements_widgets()
        self.parent.update_hkl_lambda_d0()
        self.parent.check_status_of_material_widgets()

        o_plot = Step1Plot(parent=self.parent, data_type="normalized")
        o_plot.display_general_bragg_edge()

        self.parent.list_of_element_index_changed(index="0")

        self.close()



#!/usr/bin/env python
"""
KropffGoodFitSettingsLauncher class for launching the Kropff Good Fit Settings.
"""

from qtpy.QtWidgets import QDialog

from ibeatles import load_ui


class KropffGoodFitSettingsLauncher(QDialog):
    fit_conditions: dict = {}

    def __init__(self, parent=None):
        self.parent = parent
        super(QDialog, self).__init__(parent)
        self.ui = load_ui("ui_kropff_good_fit_settings.ui", baseinstance=self)
        self.fit_conditions = self.parent.kropff_bragg_peak_good_fit_conditions
        self.rejection_conditions = self.parent.kropff_bragg_peak_row_rejections_conditions
        self.init_widgets()

    def init_widgets(self):
        self.ui.l_hkl_error_label.setText("\u03bb<sub>hkl</sub>")
        self.ui.t_error_label.setText("\u03c4")
        self.ui.sigma_error_label.setText("\u03c3")

        self.ui.l_hkl_less_than_label.setText("\u03bb<sub>hkl</sub>")
        self.ui.l_hkl_more_than_label.setText("\u03bb<sub>hkl</sub>")

        fit_conditions = self.fit_conditions
        self.ui.lambda_hkl_checkBox.setChecked(fit_conditions["l_hkl_error"]["state"])
        self.ui.lambda_hkl_doubleSpinBox.setValue(fit_conditions["l_hkl_error"]["value"])
        self.ui.tau_checkBox.setChecked(fit_conditions["t_error"]["state"])
        self.ui.tau_doubleSpinBox.setValue(fit_conditions["t_error"]["value"])
        self.ui.sigma_checkBox.setChecked(fit_conditions["sigma_error"]["state"])
        self.ui.sigma_doubleSpinBox.setValue(fit_conditions["sigma_error"]["value"])

        rejection_conditions = self.rejection_conditions
        self.ui.l_hkl_less_than_checkBox.setChecked(rejection_conditions["l_hkl"]["less_than"]["state"])
        self.ui.l_hkl_less_than_doubleSpinBox.setValue(rejection_conditions["l_hkl"]["less_than"]["value"])
        self.ui.l_hkl_more_than_checkBox.setChecked(rejection_conditions["l_hkl"]["more_than"]["state"])
        self.ui.l_hkl_more_than_doubleSpinBox.setValue(rejection_conditions["l_hkl"]["more_than"]["value"])

        self.lambda_hkl_clicked()
        self.tau_clicked()
        self.sigma_clicked()
        self.l_hkl_less_than_clicked()
        self.l_hkl_more_than_clicked()

    def ok_clicked(self):
        l_hkl_error = {
            "state": self.ui.lambda_hkl_checkBox.isChecked(),
            "value": self.ui.lambda_hkl_doubleSpinBox.value(),
        }
        t_error = {
            "state": self.ui.tau_checkBox.isChecked(),
            "value": self.ui.tau_doubleSpinBox.value(),
        }
        sigma_error = {
            "state": self.ui.sigma_checkBox.isChecked(),
            "value": self.ui.sigma_doubleSpinBox.value(),
        }

        self.parent.kropff_bragg_peak_good_fit_conditions["l_hkl_error"] = l_hkl_error
        self.parent.kropff_bragg_peak_good_fit_conditions["t_error"] = t_error
        self.parent.kropff_bragg_peak_good_fit_conditions["sigma_error"] = sigma_error

        l_hkl_less_than = {
            "state": self.ui.l_hkl_less_than_checkBox.isChecked(),
            "value": self.ui.l_hkl_less_than_doubleSpinBox.value(),
        }
        l_hkl_more_than = {
            "state": self.ui.l_hkl_more_than_checkBox.isChecked(),
            "value": self.ui.l_hkl_more_than_doubleSpinBox.value(),
        }
        self.parent.kropff_bragg_peak_row_rejections_conditions = {
            "l_hkl": {
                "less_than": l_hkl_less_than,
                "more_than": l_hkl_more_than,
            },
        }

        self.parent.update_locked_and_rejected_rows_in_bragg_peak_table()
        self.close()

    def lambda_hkl_clicked(self):
        state = self.ui.lambda_hkl_checkBox.isChecked()
        self.ui.l_hkl_label.setEnabled(state)
        self.ui.lambda_hkl_doubleSpinBox.setEnabled(state)
        self.check_ok_button()

    def tau_clicked(self):
        state = self.ui.tau_checkBox.isChecked()
        self.ui.t_label.setEnabled(state)
        self.ui.tau_doubleSpinBox.setEnabled(state)
        self.check_ok_button()

    def sigma_clicked(self):
        state = self.ui.sigma_checkBox.isChecked()
        self.ui.sigma_label.setEnabled(state)
        self.ui.sigma_doubleSpinBox.setEnabled(state)
        self.check_ok_button()

    def l_hkl_less_than_clicked(self):
        state = self.ui.l_hkl_less_than_checkBox.isChecked()
        self.ui.l_hkl_less_than_label.setEnabled(state)
        self.ui.l_hkl_label_2.setEnabled(state)
        self.ui.l_hkl_less_than_doubleSpinBox.setEnabled(state)

    def l_hkl_more_than_clicked(self):
        state = self.ui.l_hkl_more_than_checkBox.isChecked()
        self.ui.l_hkl_more_than_label.setEnabled(state)
        self.ui.l_hkl_label_3.setEnabled(state)
        self.ui.l_hkl_more_than_doubleSpinBox.setEnabled(state)

    def check_ok_button(self):
        state1 = self.ui.lambda_hkl_checkBox.isChecked()
        state2 = self.ui.tau_checkBox.isChecked()
        state3 = self.ui.sigma_checkBox.isChecked()

        if (not state1) and (not state2) and (not state3):
            button_state = False
        else:
            button_state = True

        self.ui.ok_pushButton.setEnabled(button_state)



#!/usr/bin/env python
"""
Log launcher
"""

import os

from loguru import logger
from qtpy import QtGui
from qtpy.QtGui import QIcon
from qtpy.QtWidgets import QDialog, QMainWindow

from ibeatles import load_ui, refresh_image, settings_image
from ibeatles.session import SessionSubKeys
from ibeatles.utilities.file_handler import read_ascii, write_ascii
from ibeatles.utilities.get import Get


class LogLauncher:
    def __init__(self, parent=None):
        self.parent = parent

        if self.parent.log_id is None:
            log_id = Log(parent=self.parent)
            log_id.show()
            self.parent.log_id = log_id
        else:
            self.parent.log_id.activateWindow()
            self.parent.log_id.setFocus()


class Log(QMainWindow):
    def __init__(self, parent=None):
        self.parent = parent
        QMainWindow.__init__(self, parent=parent)
        ui_full_path = os.path.join(os.path.dirname(__file__), os.path.join("ui", "log.ui"))
        self.ui = load_ui(ui_full_path, baseinstance=self)
        self.setWindowTitle("Log")
        self.ui.log_text.setReadOnly(True)

        refresh_icon = QIcon(refresh_image)
        self.ui.refresh_pushButton.setIcon(refresh_icon)

        settings_icon = QIcon(settings_image)
        self.ui.settings_pushButton.setIcon(settings_icon)

        o_get = Get(parent=self.parent)
        self.log_file_name = o_get.get_log_file_name()

        self.check_log_size()
        self.loading_logging_file()

        # jump to end of file
        self.ui.log_text.moveCursor(QtGui.QTextCursor.End)

    def closeEvent(self, c):
        self.parent.log_id = None

    def loading_logging_file(self):
        try:
            log_text = read_ascii(self.log_file_name)
            self.ui.log_text.setPlainText(log_text)
            self.ui.log_text.moveCursor(QtGui.QTextCursor.End)
        except FileNotFoundError:
            self.ui.log_text.setPlainText("")

    def clear_clicked(self):
        if os.path.exists(self.log_file_name):
            write_ascii(text="", filename=self.log_file_name)
            logger.info("log file has been cleared by user")
        self.loading_logging_file()

    def check_log_size(self):
        o_handler = LogHandler(parent=self.parent, log_file_name=self.log_file_name)
        o_handler.cut_log_size_if_bigger_than_buffer()

    def launch_settings(self):
        log_id = LogSettings(parent=self, grand_parent=self.parent)
        log_id.show()


class LogSettings(QDialog):
    def __init__(self, parent=None, grand_parent=None):
        self.parent = parent
        self.grand_parent = grand_parent
        QDialog.__init__(self, parent=self.parent)
        ui_full_path = os.path.join(os.path.dirname(__file__), os.path.join("ui", "log_settings.ui"))
        self.ui = load_ui(ui_full_path, baseinstance=self)
        self.setWindowTitle("Log")
        self.init_widgets()

    def init_widgets(self):
        log_buffer_size = self.grand_parent.session_dict[SessionSubKeys.log_buffer_size]
        self.ui.buffer_size_spinBox.setValue(log_buffer_size)

    def accept(self):
        self.grand_parent.session_dict[SessionSubKeys.log_buffer_size] = self.ui.buffer_size_spinBox.value()
        self.parent.check_log_size()
        self.parent.loading_logging_file()
        self.close()


class LogHandler:
    def __init__(self, parent=None, log_file_name=""):
        self.parent = parent
        self.log_file_name = log_file_name

    def cut_log_size_if_bigger_than_buffer(self):
        log_buffer_size = self.parent.session_dict[SessionSubKeys.log_buffer_size]
        # check current size of log file
        log_text = read_ascii(self.log_file_name)
        log_text_split_by_cr = log_text.split("\n")
        log_file_size = len(log_text_split_by_cr)
        if log_file_size <= log_buffer_size:
            return
        else:
            new_log_text = log_text_split_by_cr[-log_buffer_size:]
            new_log_text = "\n".join(new_log_text)
            write_ascii(text=new_log_text, filename=self.log_file_name)
            logger.info("log file has been truncated to fit buffer size limit")



#!/usr/bin/env python
"""
Set Fitting Variables Launcher
"""

import numpy as np
from qtpy import QtCore, QtGui
from qtpy.QtWidgets import QApplication, QMainWindow, QMenu

from ibeatles.fitting.march_dollase.event_handler import EventHandler

from .. import load_ui
from ..fitting.filling_table_handler import FillingTableHandler
from ..fitting.set_fitting_variables_handler import SetFittingVariablesHandler


class SetFittingVariablesLauncher(object):
    def __init__(self, grand_parent=None, parent=None):
        self.grand_parent = grand_parent

        if self.grand_parent.fitting_set_variables_ui is None:
            set_variables_window = SetFittingVariablesWindow(grand_parent=grand_parent, parent=parent)
            self.grand_parent.fitting_set_variables_ui = set_variables_window
            set_variables_window.show()
        else:
            self.grand_parent.fitting_set_variables_ui.setFocus()
            self.grand_parent.fitting_set_variables_ui.activateWindow()


class SetFittingVariablesWindow(QMainWindow):
    advanced_mode = True
    nbr_column = -1
    nbr_row = -1

    def __init__(self, grand_parent=None, parent=None):
        self.grand_parent = grand_parent
        self.parent = parent
        QMainWindow.__init__(self, parent=grand_parent)
        self.ui = load_ui("ui_fittingSetVariables.ui", baseinstance=self)
        # self.ui = UiMainWindow()
        # self.ui.setupUi(self)
        self.setWindowTitle("Check/Set Variables")
        self.installEventFilter(self)

        self.init_widgets()
        self.init_table()

    def eventFilter(self, object, event):
        if event.type() == QtCore.QEvent.WindowActivate:
            self.update_table()
        return False

    def init_table(self):
        fitting_selection = self.grand_parent.fitting_selection
        nbr_row = fitting_selection["nbr_row"]
        nbr_column = fitting_selection["nbr_column"]

        self.nbr_column = nbr_column
        self.nbr_row = nbr_row

        # selection table
        self.ui.variable_table.setColumnCount(nbr_column)
        self.ui.variable_table.setRowCount(nbr_row)

        # set size of cells
        value = int(self.ui.advanced_selection_cell_size_slider.value())
        self.selection_cell_size_changed(value)

    def init_widgets(self):
        self.advanced_mode = self.grand_parent.fitting_ui.ui.advanced_table_checkBox.isChecked()
        if not self.advanced_mode:
            self.ui.a5_button.setVisible(False)
            self.ui.a6_button.setVisible(False)

        self.ui.fixed_label.setStyleSheet("background-color: green")
        self.ui.locked_label.setStyleSheet("background-color: green")
        self.ui.active_label.setStyleSheet("background-color: green")

    def selection_cell_size_changed(self, value):
        nbr_row = self.ui.variable_table.rowCount()
        nbr_column = self.ui.variable_table.columnCount()

        for _row in np.arange(nbr_row):
            self.ui.variable_table.setRowHeight(_row, value)
            # self.ui.colorscale_table.setRowHeight(_row, value)

        for _col in np.arange(nbr_column):
            self.ui.variable_table.setColumnWidth(_col, value)
            # self.ui.colorscale_table.setColumnWidth(_col, value)

    def update_table(self):
        QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
        variable_selected = self.get_variable_selected()
        o_handler = SetFittingVariablesHandler(grand_parent=self.grand_parent)
        o_handler.populate_table_with_variable(variable=variable_selected)
        o_filling_table = FillingTableHandler(grand_parent=self.grand_parent, parent=self.parent)
        self.grand_parent.fitting_ui.ui.value_table.blockSignals(True)
        o_filling_table.fill_table()
        self.grand_parent.fitting_ui.ui.value_table.blockSignals(False)
        QApplication.restoreOverrideCursor()

    def get_variable_selected(self):
        if self.ui.d_spacing_button.isChecked():
            return "d_spacing"
        elif self.ui.sigma_button.isChecked():
            return "sigma"
        elif self.ui.alpha_button.isChecked():
            return "alpha"
        elif self.ui.a1_button.isChecked():
            return "a1"
        elif self.ui.a2_button.isChecked():
            return "a2"
        elif self.ui.a5_button.isChecked():
            return "a5"
        elif self.ui.a6_button.isChecked():
            return "a6"

    def apply_new_value_to_selection(self):
        variable_selected = self.get_variable_selected()
        selection = self.grand_parent.fitting_set_variables_ui.ui.variable_table.selectedRanges()
        o_handler = SetFittingVariablesHandler(grand_parent=self.grand_parent)
        new_variable = float(str(self.grand_parent.fitting_set_variables_ui.ui.new_value_text_edit.text()))
        o_handler.set_new_value_to_selected_bins(
            selection=selection,
            variable_name=variable_selected,
            variable_value=new_variable,
            table_nbr_row=self.nbr_row,
        )
        self.grand_parent.fitting_set_variables_ui.ui.new_value_text_edit.setText("")
        o_filling_table = FillingTableHandler(grand_parent=self.grand_parent, parent=self.parent)
        self.grand_parent.fitting_ui.ui.value_table.blockSignals(True)
        o_filling_table.fill_table()
        self.grand_parent.fitting_ui.ui.value_table.blockSignals(False)

    def variable_table_right_click(self, position):
        o_variable = VariableTableHandler(grand_parent=self.grand_parent)
        o_variable.right_click(position=position)

    def closeEvent(self, event=None):
        self.grand_parent.fitting_set_variables_ui = None


class VariableTableHandler(object):
    def __init__(self, grand_parent=None):
        self.grand_parent = grand_parent

    def right_click(self, position=None):
        menu = QMenu(self.grand_parent)

        _activate = menu.addAction("Activate Selection")
        _deactivate = menu.addAction("Deactivate Selection")
        menu.addSeparator()
        _lock = menu.addAction("Lock Selection")
        _unlock = menu.addAction("Unlock Selection")
        menu.addSeparator()
        _fixed = menu.addAction("Fixed Selection")
        _unfixed = menu.addAction("Unfixed Selection")

        action = menu.exec_(QtGui.QCursor.pos())

        if action == _lock:
            self.lock_selection()
        elif action == _unlock:
            self.unlock_selection()
        elif action == _activate:
            self.activate_selection()
        elif action == _deactivate:
            self.deactivate_selection()
        elif action == _fixed:
            self.fixed_selection()
        elif action == _unfixed:
            self.unfixed_selection()

    def set_fixed_status_of_selection(self, state=True):
        selection = self.grand_parent.fitting_set_variables_ui.ui.variable_table.selectedRanges()
        table_dictionary = self.grand_parent.march_table_dictionary
        nbr_row = self.grand_parent.fitting_set_variables_ui.nbr_row
        o_handler = SetFittingVariablesHandler(grand_parent=self.grand_parent)
        variable_selected = o_handler.get_variable_selected()

        for _select in selection:
            _left_column = _select.leftColumn()
            _right_column = _select.rightColumn()
            _top_row = _select.topRow()
            _bottom_row = _select.bottomRow()
            for _row in np.arange(_top_row, _bottom_row + 1):
                for _col in np.arange(_left_column, _right_column + 1):
                    _index = _row + _col * nbr_row
                    table_dictionary[str(_index)][variable_selected]["fixed"] = state

            # remove selection markers
            self.grand_parent.fitting_set_variables_ui.ui.variable_table.setRangeSelected(_select, False)

        self.grand_parent.march_table_dictionary = table_dictionary
        self.grand_parent.fitting_set_variables_ui.update_table()

    def fixed_selection(self):
        self.set_fixed_status_of_selection(state=True)

    def unfixed_selection(self):
        self.set_fixed_status_of_selection(state=False)

    def activate_selection(self):
        QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
        self.change_state_of_bins(name="active", state=True)
        self.update_fitting_ui(name="active")
        self.update_advanced_selection_ui(name="active")
        self.grand_parent.fitting_ui.update_bragg_edge_plot()
        QApplication.restoreOverrideCursor()

    def deactivate_selection(self):
        QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
        self.change_state_of_bins(name="active", state=False)
        self.update_fitting_ui(name="active")
        self.update_advanced_selection_ui(name="active")
        self.grand_parent.fitting_ui.update_bragg_edge_plot()
        QApplication.restoreOverrideCursor()

    def lock_selection(self):
        QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
        self.change_state_of_bins(name="lock", state=True)
        self.update_fitting_ui(name="lock")
        self.update_advanced_selection_ui(name="lock")
        self.grand_parent.fitting_ui.update_bragg_edge_plot()
        QApplication.restoreOverrideCursor()

    def unlock_selection(self):
        QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
        self.change_state_of_bins(name="lock", state=False)
        self.update_fitting_ui(name="lock")
        self.update_advanced_selection_ui(name="lock")
        self.grand_parent.fitting_ui.update_bragg_edge_plot()
        QApplication.restoreOverrideCursor()

    def change_state_of_bins(self, name="lock", state=True):
        selection = self.grand_parent.fitting_set_variables_ui.ui.variable_table.selectedRanges()
        table_dictionary = self.grand_parent.march_table_dictionary
        nbr_row = self.grand_parent.fitting_set_variables_ui.nbr_row

        for _select in selection:
            _left_column = _select.leftColumn()
            _right_column = _select.rightColumn()
            _top_row = _select.topRow()
            _bottom_row = _select.bottomRow()
            for _row in np.arange(_top_row, _bottom_row + 1):
                for _col in np.arange(_left_column, _right_column + 1):
                    _index = _row + _col * nbr_row
                    table_dictionary[str(_index)][name] = state

            # remove selection markers
            self.grand_parent.fitting_set_variables_ui.ui.variable_table.setRangeSelected(_select, False)

        self.grand_parent.march_table_dictionary = table_dictionary
        self.grand_parent.fitting_set_variables_ui.update_table()

    def update_fitting_ui(self, name="active"):
        o_event = EventHandler(parent=self.parent, grand_parent=self.grand_parent)
        if name == "lock":
            o_event.update_image_view_lock()
        elif name == "active":
            o_event.update_image_view_selection()

        o_filling_table = FillingTableHandler(grand_parent=self.grand_parent)
        self.grand_parent.fitting_ui.ui.value_table.blockSignals(True)
        o_filling_table.fill_table()

        self.grand_parent.fitting_ui.ui.value_table.blockSignals(False)

    def update_advanced_selection_ui(self, name="active"):
        if self.grand_parent.advanced_selection_ui:
            if name == "lock":
                self.grand_parent.advanced_selection_ui.update_lock_table()
            elif name == "active":
                self.grand_parent.advanced_selection_ui.update_selected_table()
